@@ -1,6 +1,7 @@
 #!/usr/bin/python
-
-#Benchmark for Azimuthal integration of PyFAI
+# coding: utf-8
+# author: Jérôme Kieffer
+# Benchmark for Azimuthal integration of PyFAI
 
 from __future__ import print_function, division
 
@@ -24,23 +25,21 @@ try:
 except ImportError:
     print("No socket opened for debugging -> please install rfoo")
 
-#We use the locally build version of PyFAI
+# We use the locally build version of PyFAI
 pyFAI = utilstest.UtilsTest.pyFAI
 ocl = pyFAI.opencl.ocl
 from pyFAI.gui_utils import pylab, update_fig
 
 ds_list = ["Pilatus1M.poni", "halfccd.poni", "Frelon2k.poni", "Pilatus6M.poni", "Mar3450.poni", "Fairchild.poni"]
-datasets = {"Fairchild.poni":utilstest.UtilsTest.getimage("1880/Fairchild.edf"),
-            "halfccd.poni":utilstest.UtilsTest.getimage("1882/halfccd.edf"),
-            "Frelon2k.poni":utilstest.UtilsTest.getimage("1881/Frelon2k.edf"),
-            "Pilatus6M.poni":utilstest.UtilsTest.getimage("1884/Pilatus6M.cbf"),
-            "Pilatus1M.poni":utilstest.UtilsTest.getimage("1883/Pilatus1M.edf"),
-            "Mar3450.poni":utilstest.UtilsTest.getimage("2201/LaB6_260210.mar3450")
+datasets = {"Fairchild.poni": utilstest.UtilsTest.getimage("1880/Fairchild.edf"),
+            "halfccd.poni": utilstest.UtilsTest.getimage("1882/halfccd.edf"),
+            "Frelon2k.poni": utilstest.UtilsTest.getimage("1881/Frelon2k.edf"),
+            "Pilatus6M.poni": utilstest.UtilsTest.getimage("1884/Pilatus6M.cbf"),
+            "Pilatus1M.poni": utilstest.UtilsTest.getimage("1883/Pilatus1M.edf"),
+            "Mar3450.poni": utilstest.UtilsTest.getimage("2201/LaB6_260210.mar3450")
             }
 
-
-
-#Handle to the Bench instance: allows debugging from outside if needed
+# Handle to the Bench instance: allows debugging from outside if needed
 bench = None
 
 
@@ -93,7 +92,10 @@ data = fabio.open(r"%s").data
             if os.name == "nt":
                 self._cpu = platform.processor()
             elif os.path.exists("/proc/cpuinfo"):
-                self._cpu = [i.split(": ", 1)[1] for i in open("/proc/cpuinfo") if i.startswith("model name")][0].strip()
+                cpuinfo = [i.split(": ", 1)[1] for i in open("/proc/cpuinfo") if i.startswith("model name")]
+                if not cpuinfo:
+                    cpuinfo = [i.split(": ", 1)[1] for i in open("/proc/cpuinfo") if i.startswith("cpu")]
+                self._cpu = cpuinfo[0].strip()
             elif os.path.exists("/usr/sbin/sysctl"):
                 proc = subprocess.Popen(["sysctl", "-n", "machdep.cpu.brand_string"], stdout=subprocess.PIPE)
                 proc.wait()
@@ -194,9 +196,16 @@ data = fabio.open(r"%s").data
             self.update_mp()
             if check:
                 if "lut" in method:
-                    print("lut: shape= %s \t nbytes %.3f MB " % (ai._lut_integrator.lut.shape, ai._lut_integrator.lut_nbytes / 2 ** 20))
+                    try:
+                        print("lut: shape= %s \t nbytes %.3f MB " % (ai._lut_integrator.lut.shape, ai._lut_integrator.lut_nbytes / 2 ** 20))
+                    except MemoryError as error:
+                        print(error)
                 elif "csr" in method:
-                    print("csr: size= %s \t nbytes %.3f MB " % (ai._csr_integrator.data.size, ai._csr_integrator.lut_nbytes / 2 ** 20))
+                    try:
+                        print("csr: size= %s \t nbytes %.3f MB " % (ai._csr_integrator.data.size, ai._csr_integrator.lut_nbytes / 2 ** 20))
+                    except MemoryError as error:
+                        print(error)
+
             del ai, data
             self.update_mp()
             try:
@@ -486,7 +495,7 @@ if __name__ == "__main__":
     try:
         from argparse import ArgumentParser
     except:
-        from pyFAI.argparse import ArgumentParser
+        from pyFAI.third_party.argparse import ArgumentParser
     description = """Benchmark for Azimuthal integration
     """
     epilog = """  """
