@@ -22,14 +22,20 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-version = "0.10.2"
-date = "2014-10"
+from __future__ import absolute_import, print_function, with_statement, division
 import sys, logging
 logging.basicConfig()
+from ._version import version, version_info, hexversion, date
 
 if sys.version_info < (2, 6):
     logger = logging.getLogger("pyFAI.__init__")
     logger.error("pyFAI required a python version >= 2.6")
     raise RuntimeError("pyFAI required a python version >= 2.6, now we are running: %s" % sys.version)
-from azimuthalIntegrator import AzimuthalIntegrator
+
+from . import utils
+from .azimuthalIntegrator import AzimuthalIntegrator
 load = AzimuthalIntegrator.sload
+
+def tests():
+    from . import test
+    test.run_tests()
\ No newline at end of file
