@@ -22,13 +22,13 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-from __future__ import division
+from __future__ import division, print_function, absolute_import
 __authors__ = ["Aurore Deschildre", "Jérôme Kieffer"]
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "25/09/2014"
-__status__ = "development"
+__date__ = "16/06/2015"
+__status__ = "production"
 __docformat__ = 'restructuredtext'
 
 import os
@@ -57,7 +57,7 @@ from .bilinear import Bilinear
 
 from math import sqrt, cos, sin, pow
 
-from .utils import binning #, timeit
+from .utils import binning, is_far_from_group
 
 def image_test():
     img = numpy.zeros((128 * 4, 128 * 4))
@@ -340,7 +340,7 @@ class BlobDetection(object):
                 kpx, kpy, kps, peak_val, valid = self.refine_Hessian(kpx, kpy, kps)
                 l = valid.sum()
                 self.ref_kp.append((kps, kpy, kpx))
-            print ('After refinement : %i keypoints' % l)
+            print('After refinement : %i keypoints' % l)
         else:
             peak_val = self.dogs[kps, kpy, kpx]
             l = kpx.size
@@ -485,7 +485,7 @@ class BlobDetection(object):
 #         SGX0Y2 = [0.0, 0.33333333 , 0.0 , 0.0 , -0.66666667,0.0, 0.0 , 0.33333333 , 0.0]
 
 
-        for y, x, sigma in itertools.izip(kpy, kpx, kps):
+        for y, x, sigma in zip(kpy, kpx, kps):
 
             curr_dog = self.dogs[sigma]
             prev_dog = self.dogs[sigma - 1]
@@ -519,13 +519,13 @@ class BlobDetection(object):
             dxs = (dx_next - dx_prev) / 2.0
             dys = (dy_next - dy_prev) / 2.0
 
-            print dx,dy,ds
-            print d2x,d2y,d2s,dxy,dxs,dys
+            print(dx,dy,ds)
+            print(d2x,d2y,d2s,dxy,dxs,dys)
 
             lap = numpy.array([[d2y, dxy, dys], [dxy, d2x, dxs], [dys, dxs, d2s]])
             delta = -(numpy.dot(numpy.linalg.inv(lap), [dy, dx, ds]))
-            print y,x
-            print delta
+            print(y,x)
+            print(delta)
 #                 err = numpy.linalg.norm(delta[:-1])
             if  numpy.abs(delta[0]) <= self.tresh and numpy.abs(delta[1]) <= self.tresh and numpy.abs(delta[2]) <= self.tresh:
                 k2x.append(x + delta[1])
@@ -556,7 +556,7 @@ class BlobDetection(object):
         pylab.figure()
         pylab.imshow(img, interpolation='nearest')
 
-        for y, x, s in itertools.izip(kpy, kpx, sigma):
+        for y, x, s in zip(kpy, kpx, sigma):
             s_patch = numpy.trunc(s * 2)
 
             if s_patch % 2 == 0 :
@@ -685,8 +685,8 @@ class BlobDetection(object):
 #         print "phi th"
 #         print phi_th_2
         err = numpy.sum((phi_th - phi_exp)**2)/self.keypoints.x.size
-        print "err"
-        print err
+        print("err")
+        print(err)
 
         return val,vect
 
@@ -734,7 +734,7 @@ class BlobDetection(object):
             best = self.bilinear.local_maxi(best)
         return best
 
-    def peaks_from_area(self, mask, keep=None, refine=True, Imin=None, **kwargs):
+    def peaks_from_area(self, mask, keep=None, refine=True, Imin=None, dmin=0.0, **kwargs):
         """
         Return the list of peaks within an area
 
@@ -747,6 +747,7 @@ class BlobDetection(object):
         y = numpy.round(self.keypoints.y).astype(int)
         x = numpy.round(self.keypoints.x).astype(int)
         is_inside = (mask[y, x]).astype(bool)
+        dmin2 = dmin*dmin
         if is_inside.sum() == 0:
             logger.error("No keypoint that region")
             return []
@@ -761,10 +762,18 @@ class BlobDetection(object):
                 good_kp = kp[valid]
         else:
             good_kp = kp
-        if keep and len(good_kp) > keep:
-            order = numpy.argsort(good_kp.I)
-            order = order[-1::-1][:keep] #keep only the most intense
-            good_kp = good_kp[order]
+        #sort keypoint by intensity
+        order = numpy.argsort(good_kp.I)
+        order = order[-1::-1][:keep] #keep only the most intense
+        good_kp = good_kp[order]
+        if keep and (len(good_kp) > keep):
+            keep_kp = []
+            for i in good_kp:
+                if is_far_from_group(i, keep_kp, dmin2):
+                    keep_kp.append(i)
+                if len(keep_kp)>= keep:
+                    break
+            good_kp = keep_kp
         if refine:
             if self.bilinear is None:
                 self.bilinear = Bilinear(self.raw)
@@ -830,7 +839,7 @@ if __name__ == "__main__":
 
     bd = BlobDetection(img)
     kx, ky, dx, dy, sigma = bd._one_octave()
-    print bd.sigmas
+    print(bd.sigmas)
 #
 #     #building histogram with the corrected sigmas
 #     sigma = numpy.asarray(sigma)
