@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Azimuthal integration
-#             https://github.com/kif/pyFAI
+#             https://github.com/pyFAI/pyFAI
 #
 #    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
 #
@@ -26,7 +26,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "29/09/2014"
+__date__ = "22/06/2015"
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
@@ -40,7 +40,8 @@ import types
 
 from . import detectors
 from . import units
-
+from .third_party import six
+StringTypes = (six.binary_type, six.text_type)
 
 logger = logging.getLogger("pyFAI.geometry")
 
@@ -225,7 +226,7 @@ class Geometry(object):
         self._transmission_crc = None
 
         if detector:
-            if type(detector) in types.StringTypes:
+            if isinstance(detector, StringTypes):
                 self.detector = detectors.detector_factory(detector)
             else:
                 self.detector = detector
@@ -253,7 +254,7 @@ class Geometry(object):
                            f2d["tilt"], f2d["tiltPlanRotation"]))
         return os.linesep.join(lstTxt)
 
-    def _calcCartesianPositions(self, d1, d2, poni1=None, poni2=None):
+    def _calc_cartesian_positions(self, d1, d2, poni1=None, poni2=None):
         """
         Calculate the position in cartesian coordinate (centered on the PONI)
         and in meter of a couple of coordinates.
@@ -273,8 +274,8 @@ class Geometry(object):
         if poni2 is None:
             poni2 = self.poni2
 
-        p1, p2 = self.detector.calc_cartesian_positions(d1, d2)
-        return p1 - poni1, p2 - poni2
+        p1, p2, p3 = self.detector.calc_cartesian_positions(d1, d2)
+        return p1 - poni1, p2 - poni2, p3
 
     def calc_pos_zyx(self, d0=None, d1=None, d2=None, param=None):
         """
@@ -294,11 +295,13 @@ class Geometry(object):
             param = self.param
         if (d1 is None) or (d2 is None):
             raise RuntimeError("input corrdiate d1 and d2 are mandatory")
-        p1, p2 = self._calcCartesianPositions(d1, d2, param[1], param[2])
         if d0 is None:
             L = param[0]
         else:
             L = param[0] + d0
+        p1, p2, p3 = self._calc_cartesian_positions(d1, d2, param[1], param[2])
+        if p3 is not None:
+            L = L + p3
         cosRot1 = cos(param[3])
         cosRot2 = cos(param[4])
         cosRot3 = cos(param[5])
@@ -337,33 +340,15 @@ class Geometry(object):
             if param is None:
                 param = self.param
 
-            p1, p2 = self._calcCartesianPositions(d1, d2, param[1], param[2])
-
+            p1, p2, p3 = self._calc_cartesian_positions(d1, d2, param[1], param[2])
             tmp = _geometry.calc_tth(L=param[0],
                                      rot1=param[3],
                                      rot2=param[4],
                                      rot3=param[5],
                                      pos1=p1,
-                                     pos2=p2)
-            tmp.shape = p1.shape
+                                     pos2=p2,
+                                     pos3=p3)
         else:
-#             if param is None:
-#                 param = self.param
-#             p1, p2 = self._calcCartesianPositions(d1, d2, param[1], param[2])
-#             L = param[0]
-#             cosRot1 = cos(param[3])
-#             cosRot2 = cos(param[4])
-#             cosRot3 = cos(param[5])
-#             sinRot1 = sin(param[3])
-#             sinRot2 = sin(param[4])
-#             sinRot3 = sin(param[5])
-#             t1 = p1 * cosRot2 * cosRot3 + \
-#                 p2 * (cosRot3 * sinRot1 * sinRot2 - cosRot1 * sinRot3) - \
-#                 L * (cosRot1 * cosRot3 * sinRot2 + sinRot1 * sinRot3)
-#             t2 = p1 * cosRot2 * sinRot3 + \
-#                 p2 * (cosRot1 * cosRot3 + sinRot1 * sinRot2 * sinRot3) - \
-#                 L * (-(cosRot3 * sinRot1) + cosRot1 * sinRot2 * sinRot3)
-#             t3 = p1 * sinRot2 - p2 * cosRot2 * sinRot1 + L * cosRot1 * cosRot2
             zyx = self.calc_pos_zyx(d0=None, d1=d1, d2=d2, param=param)
             t1 = zyx[1]
             t2 = zyx[2]
@@ -393,16 +378,16 @@ class Geometry(object):
                                 " without knowing wavelength !!!"))
 
         if _geometry and path == "cython":
-            p1, p2 = self._calcCartesianPositions(d1, d2,
-                                                  self._poni1, self.poni2)
+            p1, p2, p3 = self._calc_cartesian_positions(d1, d2,
+                                                      self._poni1, self.poni2)
             out = _geometry.calc_q(L=self._dist,
                                    rot1=self._rot1,
                                    rot2=self._rot2,
                                    rot3=self._rot3,
                                    pos1=p1,
                                    pos2=p2,
+                                   pos3=p3,
                                    wavelength=self.wavelength)
-            out.shape = p1.shape
         else:
             out = 4.0e-9 * numpy.pi / self.wavelength * \
                 numpy.sin(self.tth(d1=d1, d2=d2, param=param) / 2.0)
@@ -411,7 +396,7 @@ class Geometry(object):
     def rFunction(self, d1, d2, param=None, path="numpy"):
         """
         Calculates the radius value for the center of a given pixel
-        (or set of pixels) in mm
+        (or set of pixels) in m
 
         r = direct_distance  * tan( 2theta )
 
@@ -419,21 +404,21 @@ class Geometry(object):
         @type d1: scalar or array of scalar
         @param d2: position(s) in pixel in second dimension (c order)
         @type d2: scalar or array of scalar
-        @return: r in in mm
+        @return: r in in m
         @rtype: float or array of floats.
         """
         cosTilt = cos(self._rot1) * cos(self._rot2)
         directDist = self._dist / cosTilt  # in m
 
         if _geometry and path == "cython":
-            p1, p2 = self._calcCartesianPositions(d1, d2, self._poni1, self.poni2)
+            p1, p2, p3 = self._calc_cartesian_positions(d1, d2, self._poni1, self.poni2)
             out = _geometry.calc_r(L=self._dist,
                                    rot1=self._rot1,
                                    rot2=self._rot2,
                                    rot3=self._rot3,
                                    pos1=p1,
-                                   pos2=p2)
-            out.shape = p1.shape
+                                   pos2=p2,
+                                   pos3=p3)
         else:
             out = directDist * numpy.tan(self.tth(d1=d1, d2=d2, param=param))
         return out
@@ -453,10 +438,10 @@ class Geometry(object):
     def rArray(self, shape):
         """
         Generate an array of the given shape with r(i,j) for all
-        elements; r in mm.
+        elements; r in m.
 
         @param shape: expected shape
-        @return: 2d array of the given shape with radius in mm from beam stop.
+        @return: 2d array of the given shape with radius in m from beam center on detector.
         """
         if self._ra is None:
             with self._sem:
@@ -473,7 +458,7 @@ class Geometry(object):
 
     def rCornerFunct(self, d1, d2):
         """
-        Calculate the radius array for any pixel corner (in mm)
+        Calculate the radius array for any pixel corner (in m)
         """
         return self.rFunction(d1 - 0.5, d2 - 0.5)
 
@@ -521,14 +506,13 @@ class Geometry(object):
         @param path: can be "tan" (i.e via numpy) or "cython"
         @return: chi, the azimuthal angle in rad
         """
-        p1, p2 = self._calcCartesianPositions(d1, d2, self._poni1, self.poni2)
+        p1, p2, p3 = self._calc_cartesian_positions(d1, d2, self._poni1, self._poni2)
 
         if path == "cython" and _geometry:
-            tmp = _geometry.calc_chi(
-                L=self._dist,
-                rot1=self._rot1, rot2=self._rot2, rot3=self._rot3,
-                pos1=p1, pos2=p2)
-            tmp.shape = p1.shape
+            tmp = _geometry.calc_chi(L=self._dist,
+                                     rot1=self._rot1, rot2=self._rot2, rot3=self._rot3,
+                                     pos1=p1, pos2=p2, pos3=p3)
+            tmp.shape = d1.shape
         else:
             cosRot1 = cos(self._rot1)
             cosRot2 = cos(self._rot2)
@@ -537,6 +521,8 @@ class Geometry(object):
             sinRot2 = sin(self._rot2)
             sinRot3 = sin(self._rot3)
             L = self._dist
+            if p3 is not None:
+                L = L + p3
             num = p1 * cosRot2 * cosRot3 \
                 + p2 * (cosRot3 * sinRot1 * sinRot2 - cosRot1 * sinRot3) \
                 - L * (cosRot1 * cosRot3 * sinRot2 + sinRot1 * sinRot3)
@@ -825,14 +811,24 @@ class Geometry(object):
                     self._dra = delta.max(axis=-1)
         return self._dra
 
-    def cosIncidance(self, d1, d2):
+    def cosIncidance(self, d1, d2, path="cython"):
         """
         Calculate the incidence angle (alpha) for current pixels (P).
-        The poni is at incidence angle=1 so cos(alpha) = 1
+        The poni being the point of normal incidence,
+        it's incidence angle is $\{alpha} = 0$ hence $cos(\{alpha}) = 1$
 
+        @param d1: 1d or 2d set of points in pixel coord
+        @param d2:  1d or 2d set of points in pixel coord
+        @return: cosine of the incidence angle
         """
-        p1, p2 = self._calcCartesianPositions(d1, d2)
-        cosa = self._dist / numpy.sqrt(self._dist * self._dist + p1 * p1 + p2 * p2)
+        p1, p2, p3 = self._calc_cartesian_positions(d1, d2)
+        if p3 is not None:
+            logger.warning("FIXME: Disable solid angle correction for 3D detectors")
+            return numpy.ones_like(d1)
+        if path == "cython":
+            cosa = _geometry.calc_cosa(self._dist, p1, p2)
+        else:
+            cosa = self._dist / numpy.sqrt(self._dist * self._dist + p1 * p1 + p2 * p2)
         return cosa
 
     def diffSolidAngle(self, d1, d2):
@@ -873,15 +869,18 @@ class Geometry(object):
 
         return dsa
 
-    def solidAngleArray(self, shape, order=3):
-        """
-        Generate an array of the given shape with the solid angle of
-        the current element two-theta(i,j) for all elements.
-
+    def solidAngleArray(self, shape, order=3, absolute=False):
+        """Generate an array for the solid angle correction
+        given the shape of the detector.
 
+        solid_angle = cos(incidence)^3
 
         @param shape: shape of the array expected
-        @param order:
+        @param order: should be 3, power of the formula just obove
+        @param absolute: the absolute solid angle is calculated as:
+
+        SA = pix1*pix2/dist^2 * cos(incidence)^3
+
         """
         if self._dssa is None:
             if order is True:
@@ -891,11 +890,15 @@ class Geometry(object):
             self._dssa = numpy.fromfunction(self.diffSolidAngle,
                                             shape, dtype=numpy.float32)
             self._dssa_crc = crc32(self._dssa)
-        return self._dssa
+        if absolute:
+            return self._dssa * self.pixel1 * self.pixel2 / self._dist / self._dist
+        else:
+            return self._dssa
+
 
     def save(self, filename):
         """
-        Save the refined parameters.
+        Save the geometry parameters.
 
         @param filename: name of the file where to save the parameters
         @type filename: string
@@ -905,17 +908,22 @@ class Geometry(object):
                 f.write(("# Nota: C-Order, 1 refers to the Y axis,"
                          " 2 to the X axis \n"))
                 f.write("# Calibration done at %s\n" % time.ctime())
-                if self.detector.name != "Detector":
-                    f.write("Detector: %s\n" % self.detector.__class__.__name__)
-                f.write("PixelSize1: %s\n" % self.pixel1)
-                f.write("PixelSize2: %s\n" % self.pixel2)
+                detector = self.detector
+                if isinstance(detector, detectors.NexusDetector) and detector._filename:
+                    f.write("Detector: %s\n" % os.path.abspath(detector._filename))
+                elif detector.name != "Detector":
+                    f.write("Detector: %s\n" % detector.__class__.__name__)
+                f.write("PixelSize1: %s\n" % detector.pixel1)
+                f.write("PixelSize2: %s\n" % detector.pixel2)
+                if detector.splineFile:
+                    f.write("SplineFile: %s\n" % detector.splineFile)
+
                 f.write("Distance: %s\n" % self._dist)
                 f.write("Poni1: %s\n" % self._poni1)
                 f.write("Poni2: %s\n" % self._poni2)
                 f.write("Rot1: %s\n" % self._rot1)
                 f.write("Rot2: %s\n" % self._rot2)
                 f.write("Rot3: %s\n" % self._rot3)
-                f.write("SplineFile: %s\n" % self.splineFile)
                 if self._wavelength is not None:
                     f.write("Wavelength: %s\n" % self._wavelength)
         except IOError:
@@ -957,10 +965,17 @@ class Geometry(object):
             data[key] = value
         if "detector" in data:
             self.detector = detectors.detector_factory(data["detector"])
-        if "pixelsize1" in data:
-            self.detector.pixel1 = float(data["pixelsize1"])
-        if "pixelsize2" in data:
-            self.detector.pixel2 = float(data["pixelsize2"])
+        else:
+            self.detector = detectors.Detector()
+        if self.detector.force_pixel and ("pixelsize1" in data) and ("pixelsize2" in data):
+            pixel1 = float(data["pixelsize1"])
+            pixel2 = float(data["pixelsize2"])
+            self.detector = self.detector.__class__(pixel1=pixel1, pixel2=pixel2)
+        else:
+            if "pixelsize1" in data:
+                self.detector.pixel1 = float(data["pixelsize1"])
+            if "pixelsize2" in data:
+                self.detector.pixel2 = float(data["pixelsize2"])
         if "distance" in data:
             self._dist = float(data["distance"])
         if "poni1" in data:
@@ -1141,28 +1156,28 @@ class Geometry(object):
         @param BSize_2: pixel binning factor along the slowst dimention
         @param WaveLength: wavelength used
         """
-        #first define the detector
+        # first define the detector
         if splineFile:
-            #let's assume the spline file is for unbinned detectors ...
+            # let's assume the spline file is for unbinned detectors ...
             self.detector = detectors.FReLoN(splineFile)
             self.detector.binning = (int(BSize_2), int(BSize_1))
         elif PSize_1 and PSize_2:
             self.detector = detectors.Detector(PSize_2, PSize_1)
             if BSize_2 > 1 or BSize_1 > 1:
-                #set binning factor without changing pixel size
+                # set binning factor without changing pixel size
                 self.detector._binning = (int(BSize_2), int(BSize_1))
 
-        #then the geometry
+        # then the geometry
         self._dist = float(SampleDistance)
         self._poni1 = float(Center_2) * self.detector.pixel1
         self._poni2 = float(Center_1) * self.detector.pixel2
-        #This is WRONG ... correct it
+        # This is WRONG ... correct it
         self._rot1 = Rot_2 or 0
         self._rot2 = Rot_1 or 0
         self._rot3 = -(Rot_3 or 0)
         if Rot_1  or Rot_2  or Rot_3 :
             raise NotImplementedError("rotation axis not yet implemented for SPD")
-        #and finally the wavelength
+        # and finally the wavelength
         if WaveLength:
             self.wavelength = float(WaveLength)
         self.reset()
@@ -1407,6 +1422,52 @@ class Geometry(object):
             calcimage[numpy.where(mask)] = 0
         return calcimage
 
+    def __copy__(self):
+        """return a shallow copy of itself.
+        """
+        new = self.__class__(detector=self.detector)
+        # transfer numerical values:
+        numerical = ["_dist", "_poni1", "_poni2", "_rot1", "_rot2", "_rot3",
+                     "chiDiscAtPi", "_dssa_crc", "_dssa_order", "_wavelength",
+                     '_oversampling', '_correct_solid_angle_for_spline',
+                     '_polarization_factor', '_polarization_axis_offset',
+                     '_polarization_crc', '_transmission_crc', '_transmission_normal']
+        array = ["_ttha", "_dttha", "_dssa", "_chia", "_dchia", "_qa", "_dqa",
+                 "_ra", "_dra", "_corner4Da", "_corner4Dqa", "_corner4Dra",
+                 '_polarization', '_cosa', '_transmission_normal', '_transmission_corr']
+        for key in numerical + array:
+            new.__setattr__(key, self.__getattribute__(key))
+        new.param = [new._dist, new._poni1, new._poni2,
+                      new._rot1, new._rot2, new._rot3]
+        return new
+
+
+    def __deepcopy__(self):
+        """return a deep copy of itself.
+        """
+        new = self.__class__()
+        new.detector = self.detector.__deepcopy__()
+        # transfer numerical values:
+        numerical = ["_dist", "_poni1", "_poni2", "_rot1", "_rot2", "_rot3",
+                     "chiDiscAtPi", "_dssa_crc", "_dssa_order", "_wavelength",
+                     '_oversampling', '_correct_solid_angle_for_spline',
+                     '_polarization_factor', '_polarization_axis_offset',
+                     '_polarization_crc', '_transmission_crc', '_transmission_normal']
+        array = ["_ttha", "_dttha", "_dssa", "_chia", "_dchia", "_qa", "_dqa",
+                 "_ra", "_dra", "_corner4Da", "_corner4Dqa", "_corner4Dra",
+                 '_polarization', '_cosa', '_transmission_normal', '_transmission_corr']
+        for key in numerical:
+            new.__setattr__(key, self.__getattribute__(key))
+        for key in array:
+            value = self.__getattribute__(key)
+            if value is not None:
+                new.__setattr__(key, 1 * value)
+            else:
+                new.__setattr__(key, None)
+        new.param = [new._dist, new._poni1, new._poni2,
+                      new._rot1, new._rot2, new._rot3]
+        return new
+
 # ############################################
 # Accessors and public properties of the class
 # ############################################
@@ -1600,3 +1661,19 @@ class Geometry(object):
 
     correct_SA_spline = property(get_correct_solid_angle_for_spline,
                                  set_correct_solid_angle_for_spline)
+
+    def set_maskfile(self, maskfile):
+        self.detector.set_maskfile(maskfile)
+
+    def get_maskfile(self):
+        return self.detector.get_maskfile()
+
+    maskfile = property(get_maskfile, set_maskfile)
+
+    def set_mask(self, mask):
+        self.detector.set_mask(mask)
+
+    def get_mask(self):
+        return self.detector.get_mask()
+
+    mask = property(get_mask, set_mask)
