@@ -22,6 +22,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import absolute_import, print_function, with_statement, division
 
 """
 pyFAI-integrate
@@ -35,7 +36,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "29/09/2014"
+__date__ = "21/05/2015"
 __satus__ = "development"
 
 import sys
@@ -46,18 +47,22 @@ import time
 import types
 import threading
 import math
-
+import weakref
 import os.path as op
 import numpy
 logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger("pyFAI.integrate_widget")
 from .gui_utils import QtCore, QtGui, uic, SIGNAL, QtWebKit
 
-import pyFAI
 import fabio
+from .detectors import ALL_DETECTORS, detector_factory
 from .opencl import ocl
 from .utils import float_, int_, str_, get_ui_file
 from .io import HDF5Writer
+from .azimuthalIntegrator import AzimuthalIntegrator
+from .units import RADIAL_UNITS, TTH_DEG, TTH_RAD, Q_NM, Q_A, R_MM
+from .third_party import six
+
 UIC = get_ui_file("integration.ui")
 
 FROM_PYMCA = "From PyMca"
@@ -99,9 +104,9 @@ class Browser(QtGui.QMainWindow):
         self.mainLayout.addWidget(self.frame)
         self.setCentralWidget(self.centralwidget)
 
-        self.connect(self.tb_url, QtCore.SIGNAL("returnPressed()"), self.browse)
-        self.connect(self.bt_back, QtCore.SIGNAL("clicked()"), self.html.back)
-        self.connect(self.bt_ahead, QtCore.SIGNAL("clicked()"), self.html.forward)
+        self.tb_url.returnPressed.connect(self.browse)
+        self.bt_back.clicked.connect(self.html.back)
+        self.bt_ahead.clicked.connect(self.html.forward)
 
         self.default_url = default_url
         self.tb_url.setText(self.default_url)
@@ -109,24 +114,28 @@ class Browser(QtGui.QMainWindow):
 
     def browse(self):
         """
-            Make a web browse on a specific url and show the page on the
-            Webview widget.
+        Make a web browse on a specific url and show the page on the
+        Webview widget.
         """
+        print("browse " + self.tb_url.text())
+        url = QtCore.QUrl.fromUserInput(self.tb_url.text())
+        print(str(url))
+#         self.html.setUrl(url)
+        self.html.load(url)
+#         self.html.show()
 
-        url = self.tb_url.text() if self.tb_url.text() else self.default_url
-        self.html.load(QtCore.QUrl(url))
-        self.html.show()
 
 class AIWidget(QtGui.QWidget):
     """
     """
-    def __init__(self, input_data=None):
-        self.ai = pyFAI.AzimuthalIntegrator()
+    def __init__(self, input_data=None, output_path=None, output_format=None, slow_dim=None, fast_dim=None):
+        self.units = {}
+        self.ai = AzimuthalIntegrator()
         self.input_data = input_data
-        self.output_path = None
-        self.output_format = None
-        self.slow_dim = None
-        self.fast_dim = None
+        self.output_path = output_path
+        self.output_format = output_format
+        self.slow_dim = slow_dim
+        self.fast_dim = fast_dim
         self.name = None
         self._sem = threading.Semaphore()
         QtGui.QWidget.__init__(self)
@@ -135,7 +144,7 @@ class AIWidget(QtGui.QWidget):
         except AttributeError as error:
             logger.error("I looks like your installation suffers from this bug: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=697348")
             raise RuntimeError("Please upgrade your installation of PyQt (or apply the patch)")
-        self.all_detectors = pyFAI.detectors.ALL_DETECTORS.keys()
+        self.all_detectors = list(ALL_DETECTORS.keys())
         self.all_detectors.sort()
         self.detector.addItems([i.capitalize() for i in self.all_detectors])
         self.detector.setCurrentIndex(self.all_detectors.index("detector"))
@@ -159,10 +168,31 @@ class AIWidget(QtGui.QWidget):
         self.connect(self.do_OpenCL, SIGNAL("clicked()"), self.openCL_changed)
         self.connect(self.platform, SIGNAL("currentIndexChanged(int)"), self.platform_changed)
         self.set_validators()
+        self.assign_unit()
         self.restore()
         self.progressBar.setValue(0)
         self.hdf5_path = None
 
+    def assign_unit(self):
+        """
+        assign unit to the corresponding widget
+        """
+        self.units = {}
+        for unit in RADIAL_UNITS:
+            if unit.REPR == "2th_deg":
+                self.units[unit] = self.tth_deg
+            elif unit.REPR == "2th_rad":
+                self.units[unit] = self.tth_rad
+            elif unit.REPR == "q_nm^-1":
+                self.units[unit] = self.q_nm
+            elif unit.REPR == "q_A^-1":
+                self.units[unit] = self.q_A
+            elif unit.REPR == "r_mm":
+                self.units[unit] = self.r_mm
+            else:
+                logger.warning("Unit unknown to GUI %s" % unit)
+
+
     def set_validators(self):
         """
         Set all validators for text entries
@@ -172,24 +202,25 @@ class AIWidget(QtGui.QWidget):
         self.nbpt_rad.setValidator(npt_validator)
         self.nbpt_azim.setValidator(npt_validator)
 
-        wl_validator = QtGui.QDoubleValidator()
+        wl_validator = QtGui.QDoubleValidator(self)
         wl_validator.setBottom(1e-15)
         wl_validator.setTop(1e-6)
         self.wavelength.setValidator(wl_validator)
 
-        distance_validator = QtGui.QDoubleValidator()
+        distance_validator = QtGui.QDoubleValidator(self)
         distance_validator.setBottom(0)
         self.pixel1.setValidator(distance_validator)
         self.pixel2.setValidator(distance_validator)
         self.poni1.setValidator(distance_validator)
         self.poni2.setValidator(distance_validator)
 
-        angle_validator = QtGui.QDoubleValidator()
+        angle_validator = QtGui.QDoubleValidator(self)
         distance_validator.setBottom(-math.pi)
         distance_validator.setTop(math.pi)
         self.rot1.setValidator(angle_validator)
         self.rot2.setValidator(angle_validator)
         self.rot3.setValidator(angle_validator)
+        # done at widget level
 #        self.polarization_factor.setValidator(QtGui.QDoubleValidator(-1, 1, 3))
 
 
@@ -201,25 +232,17 @@ class AIWidget(QtGui.QWidget):
             self.set_ai()
 
     #        Default Keyword arguments
-            kwarg = {"unit": "2th_deg",
+            kwarg = {"unit": TTH_DEG,
                      "dummy": None,
                      "delta_dummy": None,
-                     "method": "lut",
                      "polarization_factor":None,
                      "filename": None,
                      "safe": False,
                      }
-
-            if self.q_nm.isChecked():
-                kwarg["unit"] = "q_nm^-1"
-            elif self.tth_deg.isChecked():
-                kwarg["unit"] = "2th_deg"
-            elif self.tth_rad.isChecked():
-                kwarg["unit"] = "2th_rad"
-            elif self.r_mm.isChecked():
-                kwarg["unit"] = "r_mm"
-            elif self.q_A.isChecked():
-                kwarg["unit"] = "q_A^-1"
+            for unit, widget in self.units.items():
+                if widget is not None and widget.isChecked():
+                    kwarg["unit"] = unit
+                    break
             else:
                 logger.warning("Undefined unit !!! falling back on 2th_deg")
 
@@ -245,11 +268,10 @@ class AIWidget(QtGui.QWidget):
 #                icon=QtGui.QMessageBox.Warning, buttons=QtGui.QMessageBox.Warning)
 #
 #                msgBox.exec_()
-                ret = QtGui.QMessageBox.warning(self, u"PyFAI integrate",
-                                          u"You must provide the number of output radial bins !",
-                                          )
+                ret = QtGui.QMessageBox.warning(self, "PyFAI integrate",
+                                          "You must provide the number of output radial bins !",)
                 return {}
-                #raise RuntimeError("The number of output point is undefined !")
+                # raise RuntimeError("The number of output point is undefined !")
             kwarg["npt_rad"] = int(str(self.nbpt_rad.text()).strip())
             if self.do_2D.isChecked():
                 kwarg["npt_azim"] = int(str(self.nbpt_azim.text()).strip())
@@ -259,9 +281,14 @@ class AIWidget(QtGui.QWidget):
                 pid = platform.id
                 did = platform.get_device(self.device.currentText()).id
                 if (pid is not None) and (did is not None):
-                    kwarg["method"] = "lut_ocl_%i,%i" % (pid, did)
+                    kwarg["method"] = "csr_ocl_%i,%i" % (pid, did)
                 else:
-                    kwarg["method"] = "lut_ocl"
+                    kwarg["method"] = "csr_ocl"
+            else:
+                if len(self.input_data) > 5:
+                    kwarg["method"] = "csr"
+                else:
+                    kwarg["method"] = "splitbbox"
 
             if self.do_radial_range.isChecked():
                 try:
@@ -346,7 +373,7 @@ class AIWidget(QtGui.QWidget):
                 for i, item in enumerate(self.input_data):
                     self.progressBar.setValue(100.0 * i / len(self.input_data))
                     logger.debug("processing %s" % item)
-                    if (type(item) in types.StringTypes) and op.exists(item):
+                    if isinstance(item, (six.text_type, six.binary_type)) and op.exists(item):
                         fab_img = fabio.open(item)
                         multiframe = (fab_img.nframes > 1)
                         kwarg["data"] = fab_img.data
@@ -414,11 +441,11 @@ class AIWidget(QtGui.QWidget):
         @type filename: str
 
         """
-        print "Dump!"
-        to_save = { "poni": str(self.poni.text()).strip(),
-                    "detector": str(self.detector.currentText()).lower(),
+        logger.info("Dump!")
+        to_save = { "poni": str_(self.poni.text()).strip(),
+                    "detector": str_(self.detector.currentText()).lower(),
                     "wavelength":float_(self.wavelength.text()),
-                    "splineFile":str(self.splineFile.text()).strip(),
+                    "splineFile":str_(self.splineFile.text()).strip(),
                     "pixel1": float_(self.pixel1.text()),
                     "pixel2":float_(self.pixel2.text()),
                     "dist":float_(self.dist.text()),
@@ -434,9 +461,9 @@ class AIWidget(QtGui.QWidget):
                     "do_polarization":bool(self.do_polarization.isChecked()),
                     "val_dummy":float_(self.val_dummy.text()),
                     "delta_dummy":float_(self.delta_dummy.text()),
-                    "mask_file":str(self.mask_file.text()).strip(),
-                    "dark_current":str(self.dark_current.text()).strip(),
-                    "flat_field":str(self.flat_field.text()).strip(),
+                    "mask_file":str_(self.mask_file.text()).strip(),
+                    "dark_current":str_(self.dark_current.text()).strip(),
+                    "flat_field":str_(self.flat_field.text()).strip(),
                     "polarization_factor":float_(self.polarization_factor.value()),
                     "nbpt_rad":int_(self.nbpt_rad.text()),
                     "do_2D":bool(self.do_2D.isChecked()),
@@ -451,16 +478,12 @@ class AIWidget(QtGui.QWidget):
                     "azimuth_range_min":float_(self.azimuth_range_min.text()),
                     "azimuth_range_max":float_(self.azimuth_range_max.text()),
                    }
-        if self.q_nm.isChecked():
-            to_save["unit"] = "q_nm^-1"
-        elif self.q_A.isChecked():
-            to_save["unit"] = "q_A^-1"
-        elif self.tth_deg.isChecked():
-            to_save["unit"] = "2th_deg"
-        elif self.tth_rad.isChecked():
-            to_save["unit"] = "2th_rad"
-        elif self.r_mm.isChecked():
-            to_save["unit"] = "r_mm"
+        for unit, widget in self.units.items():
+            if widget is not None and widget.isChecked():
+                to_save["unit"] = unit.REPR
+                break
+        else:
+            logger.warning("Undefined unit !!!")
         try:
             with open(filename, "w") as myFile:
                 json.dump(to_save, myFile, indent=4)
@@ -523,17 +546,10 @@ class AIWidget(QtGui.QWidget):
             if key in data:
                 value(data[key])
         if "unit" in data:
-            unit = data["unit"].lower()
-            if unit == "q_nm^-1":
-                self.q_nm.setChecked(1)
-            elif unit == "q_A^-1":
-                self.q_A.setChecked(1)
-            elif unit == "2th_deg":
-                self.tth_deg.setChecked(1)
-            elif unit == "2th_rad":
-                self.tth_rad.setChecked(1)
-            elif unit == "r_mm":
-                self.r_mm.setChecked(1)
+            for unit, widget in self.units.items():
+                if unit.REPR == data["unit"] and widget is not None:
+                    widget.setChecked(True)
+                    break
         if "detector" in data:
             detector = data["detector"].lower()
             if detector in self.all_detectors:
@@ -542,11 +558,11 @@ class AIWidget(QtGui.QWidget):
     def select_ponifile(self):
         ponifile = QtGui.QFileDialog.getOpenFileName()
         self.poni.setText(ponifile)
-        self.set_ponifile(ponifile)
+        self.set_ponifile(str_(ponifile))
 
     def select_splinefile(self):
         logger.debug("select_splinefile")
-        splinefile = str(QtGui.QFileDialog.getOpenFileName())
+        splinefile = str_(QtGui.QFileDialog.getOpenFileName())
         if splinefile:
             try:
                 self.ai.detector.set_splineFile(splinefile)
@@ -558,31 +574,35 @@ class AIWidget(QtGui.QWidget):
 
     def select_maskfile(self):
         logger.debug("select_maskfile")
-        maskfile = str(QtGui.QFileDialog.getOpenFileName())
+        maskfile = str_(QtGui.QFileDialog.getOpenFileName())
         if maskfile:
             self.mask_file.setText(maskfile or "")
             self.do_mask.setChecked(True)
 
     def select_darkcurrent(self):
         logger.debug("select_darkcurrent")
-        darkcurrent = str(QtGui.QFileDialog.getOpenFileName())
+        darkcurrent = str_(QtGui.QFileDialog.getOpenFileName())
         if darkcurrent:
             self.dark_current.setText(str_(darkcurrent))
             self.do_dark.setChecked(True)
 
     def select_flatfield(self):
         logger.debug("select_flatfield")
-        flatfield = str(QtGui.QFileDialog.getOpenFileName())
+        flatfield = str_(QtGui.QFileDialog.getOpenFileName())
         if flatfield:
             self.flat_field.setText(str_(flatfield))
             self.do_flat.setChecked(True)
 
     def set_ponifile(self, ponifile=None):
         if ponifile is None:
-            ponifile = self.poni.text()
-            print ponifile
+            ponifile = str_(self.poni.text())
+#         try:
+#             str(ponifile)
+#         except UnicodeError:
+#             ponifile = ponifile.encode("utf8")
+#         print(ponifile, type(ponifile))
         try:
-            self.ai = pyFAI.load(ponifile)
+            self.ai = AzimuthalIntegrator.sload(ponifile)
         except Exception as error:
             logger.error("file %s does not look like a poni-file, error %s" % (ponifile, error))
             return
@@ -642,11 +662,11 @@ class AIWidget(QtGui.QWidget):
         return fval
 
     def set_ai(self):
-        poni = str(self.poni.text()).strip()
+        poni = str_(self.poni.text()).strip()
         if poni and op.isfile(poni):
-            self.ai = pyFAI.load(poni)
-        detector = str(self.detector.currentText()).lower().strip() or "detector"
-        self.ai.detector = pyFAI.detectors.detector_factory(detector)
+            self.ai = AzimuthalIntegrator.sload(poni)
+        detector = str_(self.detector.currentText()).lower().strip() or "detector"
+        self.ai.detector = detector_factory(detector)
 
         wavelength = str(self.wavelength.text()).strip()
         if wavelength:
@@ -659,7 +679,7 @@ class AIWidget(QtGui.QWidget):
                     logger.warning("Wavelength is in meter ... unlikely value %s" % fwavelength)
                 self.ai.wavelength = fwavelength
 
-        splineFile = str(self.splineFile.text()).strip()
+        splineFile = str_(self.splineFile.text()).strip()
         if splineFile and op.isfile(splineFile):
             self.ai.detector.splineFile = splineFile
 
@@ -675,7 +695,7 @@ class AIWidget(QtGui.QWidget):
         if self.chi_discontinuity_at_0.isChecked():
             self.ai.setChiDiscAtZero()
 
-        mask_file = str(self.mask_file.text()).strip()
+        mask_file = str_(self.mask_file.text()).strip()
         if mask_file  and bool(self.do_mask.isChecked()):
             if op.exists(mask_file):
                 try:
@@ -686,27 +706,26 @@ class AIWidget(QtGui.QWidget):
                     self.ai.mask = mask
 #            elif mask_file==FROM_PYMCA:
 #                self.ai.mask = mask
-        dark_files = [i.strip() for i in str(self.dark_current.text()).split(",")
+        dark_files = [i.strip() for i in str_(self.dark_current.text()).split(",")
                       if op.isfile(i.strip())]
         if dark_files and bool(self.do_dark.isChecked()):
             self.ai.set_darkfiles(dark_files)
 
-        flat_files = [i.strip() for i in str(self.flat_field.text()).split(",")
+        flat_files = [i.strip() for i in str_(self.flat_field.text()).split(",")
                       if op.isfile(i.strip())]
         if flat_files and bool(self.do_flat.isChecked()):
             self.ai.set_flatfiles(flat_files)
-        print self.ai
 
     def detector_changed(self):
         logger.debug("detector_changed")
-        detector = str(self.detector.currentText()).lower()
-        inst = pyFAI.detectors.detector_factory(detector)
+        detector = str_(self.detector.currentText()).lower()
+        inst = detector_factory(detector)
         if inst.force_pixel:
             self.pixel1.setText(str(inst.pixel1))
             self.pixel2.setText(str(inst.pixel2))
             self.splineFile.setText("")
         elif self.splineFile.text():
-            splineFile = str(self.splineFile.text()).strip()
+            splineFile = str_(self.splineFile.text()).strip()
             if op.isfile(splineFile):
                 inst.set_splineFile(splineFile)
                 self.pixel1.setText(str(inst.pixel1))
