@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Fast Azimuthal Integration
-#             https://github.com/kif/
+#             https://github.com/pyFAI/pyFAI
 #
 #
 #    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
@@ -24,12 +24,13 @@
 #
 
 
+from __future__ import absolute_import, print_function, division
 
 __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "24/09/2014"
+__date__ = "20/03/2015"
 __status__ = "beta"
 __docformat__ = 'restructuredtext'
 __doc__ = """
@@ -54,8 +55,8 @@ import posixpath
 import sys
 import threading
 import time
-import types
 
+from .utils import StringTypes
 from . import units
 from . import version
 
@@ -69,7 +70,7 @@ except ImportError as error:
 else:
     try:
         h5py._errors.silence_errors()
-    except AttributeError: #old h5py
+    except AttributeError:  # old h5py
         pass
 
 
@@ -95,7 +96,16 @@ def from_isotime(text, use_tz=False):
     """
     @param text: string representing the time is iso format
     """
-    text = str(text)
+    if len(text) == 1:
+        # just in case someone sets as a list
+        text = text[0]
+    try:
+        text = text.decode("ascii")
+    except:
+        text = str(text)
+    if len(text) < 19:
+        logger.warning("Not a iso-time string: %s" % text)
+        return
     base = text[:19]
     if use_tz and len(text) == 25:
         sgn = 1 if  text[:19] == "+" else -1
@@ -114,7 +124,8 @@ def is_hdf5(filename):
     if not os.path.exists(filename):
         raise IOError("No such file %s" % (filename))
     with open(filename, "rb") as f:
-        sig = [ord(i) for i in f.read(8)]
+        raw = f.read(8)
+    sig = [ord(i) for i in raw] if sys.version_info[0] < 3 else [int(i) for i in raw]
     return sig == signature
 
 
@@ -177,7 +188,7 @@ class Writer(object):
         Sets the JSON configuration
         """
 
-        if type(json_config) in types.StringTypes:
+        if type(json_config) in StringTypes:
             if os.path.isfile(json_config):
                 config = json.load(open(json_config, "r"))
             else:
@@ -237,14 +248,14 @@ class HDF5Writer(Writer):
         logger.debug("in init")
         Writer.init(self, fai_cfg, lima_cfg)
         with self._sem:
-            #TODO: this is Debug statement
+            # TODO: this is Debug statement
             open("fai_cfg.json", "w").write(json.dumps(self.fai_cfg, indent=4))
             open("lima_cfg.json", "w").write(json.dumps(self.lima_cfg, indent=4))
             self.fai_cfg["nbpt_rad"] = self.fai_cfg.get("nbpt_rad", 1000)
             if h5py:
                 try:
                     self.hdf5 = h5py.File(self.filename)
-                except IOError: #typically a corrupted HDF5 file !
+                except IOError:  # typically a corrupted HDF5 file !
                     os.unlink(self.filename)
                     self.hdf5 = h5py.File(self.filename)
             else:
@@ -322,9 +333,9 @@ class HDF5Writer(Writer):
             name = "Mapping " if self.fast_scan_width else "Scanning "
             name += "2D" if self.fai_cfg.get("nbpt_azim", 0) > 1 else "1D"
             name += " experiment"
-            self.group["title"] = name
-            self.group["program"] = "PyFAI"
-            self.group["start_time"] = get_isotime()
+            self.group["title"] = numpy.string_(name)
+            self.group["program"] = numpy.string_("PyFAI")
+            self.group["start_time"] = numpy.string_(get_isotime())
 
     def flush(self, radial=None, azimuthal=None):
         """
@@ -619,10 +630,17 @@ class Nexus(object):
             entry["end_time"] = end_time
         self.h5.close()
 
+    # Context manager for "with" statement compatibility
+    def __enter__(self, *arg, **kwarg):
+        return self
+
+    def __exit__(self, *arg, **kwarg):
+        self.close()
+
     def get_entry(self, name):
         """
         Retrieves an entry from its name
-        
+
         @param name: name of the entry to retrieve
         @return: HDF5 group of NXclass == NXentry
         """
@@ -638,7 +656,7 @@ class Nexus(object):
     def get_entries(self):
         """
         retrieves all entry sorted the latest first.
-        
+
         @return: list of HDF5 groups
         """
         entries = [(grp, from_isotime(self.h5[grp + "/start_time"].value))
@@ -647,7 +665,7 @@ class Nexus(object):
                         "start_time" in self.h5[grp] and  \
                         "NX_class" in self.h5[grp].attrs and \
                         self.h5[grp].attrs["NX_class"] == "NXentry")]
-        entries.sort(cmp=lambda a, b: 1 if a[1] < b[1] else -1) #sort entries in decreasing time
+        entries.sort(key=lambda a: a[1], reverse=True)  # sort entries in decreasing time
         return [self.h5[i[0]] for i in entries]
 
     def find_detector(self, all=False):
@@ -658,7 +676,7 @@ class Nexus(object):
         """
         result = []
         for entry in self.get_entries():
-            for instrument in self.get_class(entry, "NXsubentry"):
+            for instrument in self.get_class(entry, "NXsubentry") + self.get_class(entry, "NXinstrument"):
                 for detector in self.get_class(instrument, "NXdetector"):
                     if all:
                         result.append(detector)
@@ -666,19 +684,25 @@ class Nexus(object):
                         return detector
         return result
 
-    def new_entry(self, entry="entry"):
+    def new_entry(self, entry="entry", program_name="pyFAI", title="description of experiment", force_time=None):
         """
         Create a new entry
 
         @param entry: name of the entry
+        @param program_name: value of the field as string
+        @param title: value of the field as string
+        @force_time: enforce the start_time (as string!)
         @return: the corresponding HDF5 group
         """
         nb_entries = len(self.get_entries())
         entry_grp = self.h5.require_group("%s_%04i" % (entry, nb_entries))
         entry_grp.attrs["NX_class"] = "NXentry"
-        entry_grp["start_time"] = numpy.string_(get_isotime())
-        entry_grp["title"] = numpy.string_("description of experiment")
-        entry_grp["program_name"] = numpy.string_("pyFAI")
+        entry_grp["title"] = numpy.string_(title)
+        entry_grp["program_name"] = numpy.string_(program_name)
+        if force_time:
+            entry_grp["start_time"] = numpy.string_(force_time)
+        else:
+            entry_grp["start_time"] = numpy.string_(get_isotime())
         self.to_close.append(entry_grp)
         return entry_grp
 
@@ -690,7 +714,7 @@ class Nexus(object):
             entry = self.new_entry(entry)
         return self.new_class(entry, instrument_name, "NXinstrument")
 #        howto external link
-        #myfile['ext link'] = h5py.ExternalLink("otherfile.hdf5", "/path/to/resource")
+        # myfile['ext link'] = h5py.ExternalLink("otherfile.hdf5", "/path/to/resource")
 
     def new_class(self, grp, name, class_type="NXcollection"):
         """
@@ -745,3 +769,35 @@ class Nexus(object):
                    "NX_class" in grp[name].attrs and \
                    grp[name].attrs["NX_class"] == class_type)]
         return coll
+
+    def deep_copy(self, name, obj, where="/", toplevel=None, excluded=None, overwrite=False):
+        """
+        perform a deep copy:
+        create a "name" entry in self containing a copy of the object
+        
+        @param where: path to the toplevel object (i.e. root)
+        @param  toplevel: firectly the top level Group
+        @param excluded: list of keys to be excluded
+        @param overwrite: replace content if already existing
+        """
+        if (excluded is not None) and (name in excluded):
+            return
+        if not toplevel:
+            toplevel = self.h5[where]
+        if isinstance(obj, h5py.Group):
+            if not name in toplevel:
+                grp = toplevel.require_group(name)
+                for k, v in obj.attrs.items():
+                        grp.attrs[k] = v
+        elif isinstance(obj, h5py.Dataset):
+            if name in toplevel:
+                if overwrite:
+                    del toplevel[name]
+                    logger.warning("Overwriting %s in %s" % (toplevel[name].name, self.filename))
+                else:
+                    logger.warning("Not overwriting %s in %s" % (toplevel[name].name, self.filename))
+                    return
+            toplevel[name] = obj.value
+            for k, v in obj.attrs.items():
+                toplevel[name].attrs[k] = v
+
