@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Azimuthal integration
-#             https://github.com/kif/pyFAI
+#             https://github.com/pyFAI
 #
 #    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
 #
@@ -23,7 +23,7 @@
 
 __author__ = "Jerome Kieffer"
 __license__ = "GPLv3"
-__date__ = "10/10/2014"
+__date__ = "14/03/2015"
 __copyright__ = "2012, ESRF, Grenoble"
 __contact__ = "jerome.kieffer@esrf.fr"
 
@@ -33,22 +33,19 @@ import hashlib
 import numpy
 from .opencl import ocl, pyopencl, allocate_cl_buffers, release_cl_buffers
 from .splitBBoxLUT import HistoBBox1d
-from .utils import concatenate_cl_kernel
+from .utils import concatenate_cl_kernel, calc_checksum
 if pyopencl:
     mf = pyopencl.mem_flags
 else:
     raise ImportError("pyopencl is not installed")
-try:
-    from .fastcrc import crc32
-except:
-    from zlib import crc32
 logger = logging.getLogger("pyFAI.ocl_azim_lut")
 
 class OCL_LUT_Integrator(object):
     BLOCK_SIZE = 16
     def __init__(self, lut, image_size, devicetype="all",
                  platformid=None, deviceid=None,
-                 checksum=None, profile=False):
+                 checksum=None, profile=False,
+                 empty=None):
         """
         @param lut: array of int32 - float32 with shape (nbins, lut_size) with indexes and coefficients
         @param image_size: Expected image size: image.shape.prod()
@@ -59,6 +56,7 @@ class OCL_LUT_Integrator(object):
         @type deviceid: int
         @param checksum: pre - calculated checksum to prevent re - calculating it :)
         @param profile: store profiling elements
+        @param empty: value to be assigned to bins without contribution from any pixel
         """
         self._sem = threading.Semaphore()
         self._lut = lut
@@ -66,6 +64,8 @@ class OCL_LUT_Integrator(object):
         self.bins, self.lut_size = lut.shape
         self.size = image_size
         self.profile = profile
+        self.empty = empty or 0.0
+
         if not checksum:
             checksum = crc32(self._lut)
         self.on_device = {"lut":checksum, "dark":None, "flat":None, "polarization":None, "solidangle":None}
@@ -77,7 +77,7 @@ class OCL_LUT_Integrator(object):
             if res:
                 platformid, deviceid = res
             else:
-                logger.warning("No such devicetype %s"%devicetype)
+                logger.warning("No such devicetype %s" % devicetype)
                 platformid, deviceid = ocl.select_device()
         elif platformid is None:
             platformid = 0
@@ -204,7 +204,26 @@ class OCL_LUT_Integrator(object):
 
     def integrate(self, data, dummy=None, delta_dummy=None, dark=None, flat=None, solidAngle=None, polarization=None,
                             dark_checksum=None, flat_checksum=None, solidAngle_checksum=None, polarization_checksum=None,
-                            preprocess_only=False):
+                            preprocess_only=False, safe=True):
+        """
+        Before performing azimuthal integration, the preprocessing is :
+        
+        data = (data - dark) / (flat*solidAngle*polarization)
+        
+        Integration is performed using the CSR representation of the look-up table
+        
+        @param dark: array of same shape as data for pre-processing
+        @param flat: array of same shape as data for pre-processing
+        @param solidAngle: array of same shape as data for pre-processing
+        @param polarization: array of same shape as data for pre-processing
+        @param dark_checksum: CRC32 checksum of the given array
+        @param flat_checksum: CRC32 checksum of the given array
+        @param solidAngle_checksum: CRC32 checksum of the given array
+        @param polarization_checksum: CRC32 checksum of the given array
+        @param safe: if True (default) compares arrays on GPU according to their checksum, unless, use the buffer location is used
+        @param preprocess_only: return the dark subtracted; flat field & solidAngle & polarization corrected image, else
+        @return averaged data, weighted histogram, unweighted histogram
+        """
         events = []
         with self._sem:
             if data.dtype == numpy.uint8:
@@ -241,13 +260,13 @@ class OCL_LUT_Integrator(object):
                 do_dummy = numpy.int32(1)
                 dummy = numpy.float32(dummy)
                 if delta_dummy == None:
-                    delta_dummy = numpy.float32(0)
+                    delta_dummy = numpy.float32(0.0)
                 else:
                     delta_dummy = numpy.float32(abs(delta_dummy))
             else:
                 do_dummy = numpy.int32(0)
-                dummy = numpy.float32(0)
-                delta_dummy = numpy.float32(0)
+                dummy = numpy.float32(self.empty)
+                delta_dummy = numpy.float32(0.0)
             self._cl_kernel_args["corrections"][9] = do_dummy
             self._cl_kernel_args["corrections"][10] = dummy
             self._cl_kernel_args["corrections"][11] = delta_dummy
@@ -257,7 +276,7 @@ class OCL_LUT_Integrator(object):
             if dark is not None:
                 do_dark = numpy.int32(1)
                 if not dark_checksum:
-                    dark_checksum = crc32(dark)
+                    dark_checksum = calc_checksum(dark, safe)
                 if dark_checksum != self.on_device["dark"]:
                     ev = pyopencl.enqueue_copy(self._queue, self._cl_mem["dark"], numpy.ascontiguousarray(dark, dtype=numpy.float32))
                     events.append(("copy dark", ev))
@@ -268,7 +287,7 @@ class OCL_LUT_Integrator(object):
             if flat is not None:
                 do_flat = numpy.int32(1)
                 if not flat_checksum:
-                    flat_checksum = crc32(flat)
+                    flat_checksum = calc_checksum(flat, safe)
                 if self.on_device["flat"] != flat_checksum:
                     ev = pyopencl.enqueue_copy(self._queue, self._cl_mem["flat"], numpy.ascontiguousarray(flat, dtype=numpy.float32))
                     events.append(("copy flat", ev))
@@ -280,7 +299,7 @@ class OCL_LUT_Integrator(object):
             if solidAngle is not None:
                 do_solidAngle = numpy.int32(1)
                 if not solidAngle_checksum:
-                    solidAngle_checksum = crc32(solidAngle)
+                    solidAngle_checksum = calc_checksum(solidAngle, safe)
                 if solidAngle_checksum != self.on_device["solidangle"]:
                     ev = pyopencl.enqueue_copy(self._queue, self._cl_mem["solidangle"], numpy.ascontiguousarray(solidAngle, dtype=numpy.float32))
                     events.append(("copy solidangle", ev))
@@ -292,7 +311,7 @@ class OCL_LUT_Integrator(object):
             if polarization is not None:
                 do_polarization = numpy.int32(1)
                 if not polarization_checksum:
-                    polarization_checksum = crc32(polarization)
+                    polarization_checksum = calc_checksum(polarization, safe)
                 if polarization_checksum != self.on_device["polarization"]:
                     ev = pyopencl.enqueue_copy(self._queue, self._cl_mem["polarization"], numpy.ascontiguousarray(polarization, dtype=numpy.float32))
                     events.append(("copy polarization", ev))
