@@ -21,7 +21,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # #######################################################################################
 
-"""
+
+from __future__ import print_function, division
+__doc__ = """
 This is piece of software aims at manipulating spline files
 describing for geometric corrections of the 2D detectors using cubic-spline.
 
@@ -31,6 +33,7 @@ Mainly used at ESRF with FReLoN CCD camera.
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@esrf.eu"
 __license__ = "GPLv3+"
+__date__ = "05/03/2015"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
 
 import os
@@ -41,7 +44,11 @@ import scipy
 import logging
 import scipy.optimize
 import scipy.interpolate
-from scipy.interpolate import fitpack
+try:
+    # multithreaded version in Cython: about 2x faster on large array evaluation
+    from . import _bispev as fitpack
+except ImportError:
+    from scipy.interpolate import fitpack
 import traceback
 logger = logging.getLogger("pyFAI.spline")
 
@@ -172,7 +179,7 @@ class Spline(object):
                     databloc = []
                     for line in stringSpline[indexLine + 2:]:
                         if len(line) > 0:
-                            for i in range(len(line) / self.lenStrFloat):
+                            for i in range(len(line) // self.lenStrFloat):
                                 databloc.append(float(line[i * self.lenStrFloat: (i + 1) * self.lenStrFloat]))
                         else:
                             break
@@ -185,7 +192,7 @@ class Spline(object):
                     databloc = []
                     for line in stringSpline[indexLine + 2:]:
                         if len(line) > 0:
-                            for i in range(len(line) / self.lenStrFloat):
+                            for i in range(len(line) // self.lenStrFloat):
                                 databloc.append(float(line[i * self.lenStrFloat:(i + 1) * self.lenStrFloat]))
                         else:
                             break
@@ -283,7 +290,7 @@ class Spline(object):
                       ((intermediateTime - startTime),
                        (time.time() - intermediateTime)))
 
-    def splineFuncX(self, x, y):
+    def splineFuncX(self, x, y, list_of_points=False):
         """
         Calculates the displacement matrix using fitpack for the X
         direction on the given grid.
@@ -292,7 +299,7 @@ class Spline(object):
         @type x: ndarray
         @param y: points of the grid  in the y direction
         @type y: ndarray
-
+        @param list_of_points: if true, consider the zip(x,y) instead of the of the square array
         @return: displacement matrix for the X direction
         @rtype: ndarray
         """
@@ -303,18 +310,29 @@ class Spline(object):
             elif abs(x[:, 1:] - x[:, :-1] - numpy.zeros((x.shape[0], x.shape[1] - 1))).max() < 1e-6:
                 x = x[:, 0]
                 y = y[0]
-
+        if list_of_points and x.ndim == 1 and len(x) == len(y):
+            lx = ly = len(x)
+            x_order = x.argsort()
+            y_order = y.argsort()
+            x = x[x_order]
+            y = y[y_order]
+            x_unordered = numpy.zeros(lx, dtype=int)
+            y_unordered = numpy.zeros(ly, dtype=int)
+            x_unordered[x_order] = numpy.arange(lx)
+            y_unordered[y_order] = numpy.arange(ly)
         xDispArray = fitpack.bisplev(
             x, y, [self.xSplineKnotsX,
                    self.xSplineKnotsY,
                    self.xSplineCoeff,
                    self.splineOrder,
                    self.splineOrder ],
-            dx=0, dy=0).transpose()
+            dx=0, dy=0)
+        if list_of_points and x.ndim == 1:
+            return xDispArray[x_unordered, y_unordered]
+        else:
+            return xDispArray.T
 
-        return xDispArray
-
-    def splineFuncY(self, x, y):
+    def splineFuncY(self, x, y, list_of_points=False):
         """
         calculates the displacement matrix using fitpack for the Y
         direction
@@ -323,7 +341,7 @@ class Spline(object):
         @type x: ndarray
         @param y: points in the y direction
         @type y: ndarray
-
+        @param list_of_points: if true, consider the zip(x,y) instead of the of the square array
         @return: displacement matrix for the Y direction
         @rtype: ndarray
         """
@@ -335,15 +353,28 @@ class Spline(object):
                 x = x[:, 0]
                 y = y[0]
 
+        if list_of_points and x.ndim == 1 and len(x) == len(y):
+            lx = ly = len(x)
+            x_order = x.argsort()
+            y_order = y.argsort()
+            x = x[x_order]
+            y = y[y_order]
+            x_unordered = numpy.zeros(lx, dtype=int)
+            y_unordered = numpy.zeros(ly, dtype=int)
+            x_unordered[x_order] = numpy.arange(lx)
+            y_unordered[y_order] = numpy.arange(ly)
+
         yDispArray = fitpack.bisplev(
             x, y, [self.ySplineKnotsX,
                    self.ySplineKnotsY,
                    self.ySplineCoeff,
                    self.splineOrder,
                    self.splineOrder ],
-            dx=0, dy=0).transpose()
-
-        return yDispArray
+            dx=0, dy=0)
+        if list_of_points and x.ndim == 1:
+            return yDispArray[x_unordered, y_unordered]
+        else:
+            return yDispArray.T
 
     def array2spline(self, smoothing=1000, timing=False):
         """
@@ -621,6 +652,63 @@ class Spline(object):
                                  dx=0, dy=0)
         return delta0 + pos[0], delta1 + pos[1]
 
+
+
+    def flipud(self):
+        """
+        Flip the spline up-down
+        @return: new spline object
+        """
+        self.spline2array()
+        other = self.__class__()
+        other.xmin = self.xmin
+        other.ymin = self.ymin
+        other.xmax = self.xmax
+        other.ymax = self.ymax
+        other.xDispArray = numpy.flipud(self.xDispArray)
+        other.yDispArray = -numpy.flipud(self.yDispArray)
+        other.pixelSize = self.pixelSize
+        other.grid = self.grid
+        other.array2spline()
+        return other
+
+    def fliplr(self):
+        """
+        Flip the spline 
+        @return: new spline object
+        """
+        self.spline2array()
+        other = self.__class__()
+        other.xmin = self.xmin
+        other.ymin = self.ymin
+        other.xmax = self.xmax
+        other.ymax = self.ymax
+        other.xDispArray = -numpy.fliplr(self.xDispArray)
+        other.yDispArray = numpy.fliplr(self.yDispArray)
+        other.pixelSize = self.pixelSize
+        other.grid = self.grid
+        other.array2spline()
+        return other
+
+    def fliplrud(self):
+        """
+        Flip the spline left-right and up-down
+        @return: new spline object
+        """
+        self.spline2array()
+        other = self.__class__()
+        other.xmin = self.xmin
+        other.ymin = self.ymin
+        other.xmax = self.xmax
+        other.ymax = self.ymax
+        other.xDispArray = -numpy.flipud(numpy.fliplr(self.xDispArray))
+        other.yDispArray = -numpy.flipud(numpy.fliplr(self.yDispArray))
+        other.pixelSize = self.pixelSize
+        other.grid = self.grid
+        other.array2spline()
+        return other
+
+
 def main():
     """
     Some tests ....
@@ -634,7 +722,7 @@ def main():
         if os.path.isfile(keyword):
             spline_file = keyword
         elif keyword.lower().find("center=") in [0, 1, 2]:
-            center = map(float, keyword.split("=")[1].split("x"))
+            center = [float(i) for i in  keyword.split("=")[1].split("x")]
         elif keyword.lower().find("dist=") in [0, 1, 2]:
             distance = float(keyword.split("=")[1])
         elif keyword.lower().find("tilt=") in [0, 1, 2]:
