@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Fast Azimuthal integration
-#             https://github.com/kif/pyFAI
+#             https://github.com/pyFAI/pyFAI
 #
 #    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
 #
@@ -32,22 +32,31 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "21/10/2014"
+__date__ = "22/07/2015"
 __status__ = "production"
 
-import logging, sys, types, os, glob
+import logging
+import sys
+import types
+import os
+import glob
 import threading
+import math
 sem = threading.Semaphore()  # global lock for image processing initialization
 import numpy
 import fabio
+if "version_info" in dir(fabio) and fabio.version_info >= (0, 2, 2):
+    from fabio.nexus import exists
+else:
+    from os.path import  exists
 from scipy import ndimage
 from scipy.interpolate import interp1d
 from math import ceil, sin, cos, atan2, pi
-
+from .third_party import six
 try:
-    from . import relabel as relabelCython
+    from . import relabel as _relabel
 except:
-    relabelCython = None
+    _relabel = None
 try:
     from .directories import data_dir
 except ImportError:
@@ -58,10 +67,14 @@ import scipy.ndimage.filters
 logger = logging.getLogger("pyFAI.utils")
 import time
 timelog = logging.getLogger("pyFAI.timeit")
+depreclog = logging.getLogger("DEPRECATION")
 
 cu_fft = None  # No cuda here !
 if sys.platform != "win32":
     WindowsError = RuntimeError
+
+win32 = (os.name == "nt") and (tuple.__itemsize__ == 4)
+
 has_fftw3 = None
 try:
     import fftw3
@@ -70,6 +83,8 @@ except (ImportError, WindowsError) as err:
     logger.warn("Exception %s: FFTw3 not available. Falling back on Scipy", err)
     has_fftw3 = False
 
+EPS32 = (1.0 + numpy.finfo(numpy.float32).eps)
+
 import traceback
 
 def deprecated(func):
@@ -77,10 +92,32 @@ def deprecated(func):
         """
         decorator that deprecates the use of a function
         """
-        logger.warning("%s is Deprecated !!! %s" % (func.func_name, os.linesep.join([""] + traceback.format_stack()[:-1])))
+        name = func.func_name if sys.version_info[0] < 3 else func.__name__
+        depreclog.warning("%s is Deprecated !!! %s" % (name, os.linesep.join([""] + traceback.format_stack()[:-1])))
         return func(*arg, **kw)
     return wrapper
 
+
+StringTypes = (six.binary_type, six.text_type)
+try:
+    input = raw_input
+except NameError:
+    pass
+
+try:
+    from .fastcrc import crc32
+except:
+    from zlib import crc32
+
+def calc_checksum(ary, safe=True):
+    """
+    Calculate the checksum by default (or returns its buffer location if unsafe)
+    """
+    if safe:
+        return crc32(ary)
+    else:
+        return ary.__array_interface__['data'][0]
+
 def gaussian(M, std):
     """
     Return a Gaussian window of length M with standard-deviation std.
@@ -125,7 +162,11 @@ def str_(val):
     """
     s = ""
     if val != None:
-        s = str(val)
+        try:
+            s = str(val)
+        except UnicodeError:
+            # Python2 specific...
+            s = unicode(val)
     return s
 
 
@@ -136,16 +177,43 @@ def timeit(func):
         t1 = time.time()
         res = func(*arg, **kw)
         t2 = time.time()
-        if "func_name" in dir(func):
-            name = func.func_name
-        else:
-            name = str(func)
+        name = func.func_name if sys.version_info[0] < 3 else func.__name__
         timelog.warning("%s took %.3fs" % (name, t2 - t1))
         return res
     wrapper.__name__ = func.__name__
     wrapper.__doc__ = func.__doc__
     return wrapper
 
+
+def expand2d(vect, size2, vertical=True):
+    """
+    This expands a vector to a 2d-array.
+
+    The resul is the same as
+    if vertical:
+        numpy.outer(numpy.ones(size2), vect)
+    else:
+         numpy.outer(vect, numpy.ones( size2))
+
+    This is a ninja optimization: replace *1 with a memcopy, saves 50% of time at the ms level.
+
+    @param vect: 1d vector
+    @param size2: size
+    @param vertical: if False,
+    """
+    size1 = vect.size
+    if vertical:
+        out = numpy.empty((size2, size1), vect.dtype)
+        q = vect.reshape(1, -1)
+        q.strides = 0, vect.strides[0]
+    else:
+        out = numpy.empty((size1, size2), vect.dtype)
+        q = vect.reshape(-1, 1)
+        q.strides = vect.strides[0], 0
+    out[:, :] = q
+    return out
+
+
 def gaussian_filter(input_img, sigma, mode="reflect", cval=0.0):
     """
     2-dimensional Gaussian filter implemented with FFTw
@@ -359,10 +427,10 @@ def expand(input_img, sigma, mode="constant", cval=0.0):
         output[:k0, s1 + k1:] = input_img[0, -1]
         output[s0 + k0:, :k1] = input_img[-1, 0]
     # 4 sides
-        output[k0:k0 + s0, :k1] = numpy.outer(input_img[:, 0], numpy.ones(k1))
-        output[:k0, k1:k1 + s1] = numpy.outer(numpy.ones(k0), input_img[0, :])
-        output[-k0:, k1:s1 + k1] = numpy.outer(numpy.ones(k0), input_img[-1, :])
-        output[k0:s0 + k0, -k1:] = numpy.outer(input_img[:, -1], numpy.ones(k1))
+        output[k0:k0 + s0, :k1] = expand2d(input_img[:, 0], k1, False)
+        output[:k0, k1:k1 + s1] = expand2d(input_img[0, :], k0)
+        output[-k0:, k1:s1 + k1] = expand2d(input_img[-1, :], k0)
+        output[k0:s0 + k0, -k1:] = expand2d(input_img[:, -1], k1, False)
     elif mode == "wrap":
         # 4 corners
         output[s0 + k0:, s1 + k1:] = input_img[:k0, :k1]
@@ -394,9 +462,9 @@ def relabel(label, data, blured, max_size=None):
     @param max_size: the max number of label wanted
     @return array like label
     """
-    if relabelCython:
+    if _relabel:
         max_label = label.max()
-        a, b, c, d = relabelCython.countThem(label, data, blured)
+        a, b, c, d = _relabel.countThem(label, data, blured)
         count = d
         sortCount = count.argsort()
         invSortCount = sortCount[-1::-1]
@@ -409,15 +477,17 @@ def relabel(label, data, blured, max_size=None):
         logger.warning("relabel Cython module is not available...")
         return label
 
-def averageDark(lstimg, center_method="mean", cutoff=None):
+def averageDark(lstimg, center_method="mean", cutoff=None, quantiles=(0.5, 0.5)):
     """
     Averages a serie of dark (or flat) images.
     Centers the result on the mean or the median ...
     but averages all frames within  cutoff*std
 
     @param lstimg: list of 2D images or a 3D stack
-    @param center_method: is the center calculated by a "mean" or a "median"
+    @param center_method: is the center calculated by a "mean" or a "median", or "quantile"
     @param cutoff: keep all data where (I-center)/std < cutoff
+    @param quantiles: 2-tuple of floats average out data between the two quantiles
+
     @return: 2D image averaged
     """
     if "ndim" in dir(lstimg) and lstimg.ndim == 3:
@@ -431,11 +501,18 @@ def averageDark(lstimg, center_method="mean", cutoff=None):
             return lstimg[0].astype(numpy.float32)
         stack = numpy.zeros((length, shape[0], shape[1]), dtype=numpy.float32)
         for i, img in enumerate(lstimg):
-           stack[i] = img
+            stack[i] = img
     if center_method in dir(stack):
         center = stack.__getattribute__(center_method)(axis=0)
     elif center_method == "median":
+        logger.info("Filtering data (median)")
         center = numpy.median(stack, axis=0)
+    elif center_method.startswith("quantil"):
+        logger.info("Filtering data (quantiles)")
+        sorted_ = numpy.sort(stack, axis=0)
+        lower = max(0, int(min(quantiles) * length))
+        upper = min(length, int(max(quantiles) * length) + 1)
+        center = sorted_[lower:upper].mean(axis=0)
     else:
         raise RuntimeError("Cannot understand method: %s in averageDark" % center_method)
     if cutoff is None or cutoff <= 0:
@@ -455,7 +532,7 @@ def averageDark(lstimg, center_method="mean", cutoff=None):
 
 def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=None,
                    darks=None, flats=None, filter_="mean", correct_flat_from_dark=False,
-                   cutoff=None, format="edf"):
+                   cutoff=None, quantiles=None, fformat="edf"):
     """
     Takes a list of filenames and create an average frame discarding all saturated pixels.
 
@@ -466,13 +543,15 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
     @param maximum: maximum valid value
     @param darks: list of dark current images for subtraction
     @param flats: list of flat field images for division
-    @param filter_: can be maximum, mean or median (default=mean)
+    @param filter_: can be "min", "max", "median", "mean", "sum", "quantiles" (default='mean')
     @param correct_flat_from_dark: shall the flat be re-corrected ?
     @param cutoff: keep all data where (I-center)/std < cutoff
+    @param quantiles: 2-tuple containing the lower and upper quantile (0<q<1) to average out.
+    @param fformat: file format of the output image, default: edf
     @return: filename with the data or the data ndarray in case format=None
     """
-    if filter_ not in ["min", "max", "median", "mean"]:
-        logger.warning("Filter %s not understood. switch to mean filter")
+    if filter_ not in ["min", "max", "median", "mean", "sum", "quantiles"]:
+        logger.warning("Filter %s not understood. switch to mean filter" % filter_)
         filter_ = "mean"
     ld = len(listImages)
     sumImg = None
@@ -482,7 +561,7 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
     flat = None
     big_img = None
     for idx, fn in enumerate(listImages[:]):
-        if type(fn) in types.StringTypes:
+        if isinstance(fn, six.string_types):
             logger.info("Reading %s" % fn)
             ds = fabio.open(fn).data
         else:
@@ -494,15 +573,15 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
         if do_dark and (dark is None):
             if "ndim" in dir(darks) and darks.ndim == 3:
                 dark = averageDark(darks, center_method="mean", cutoff=4)
-            elif ("__len__" in dir(darks)) and (type(darks[0]) in types.StringTypes):
-                dark = averageDark([fabio.open(f).data for f in darks if os.path.exists(f)], center_method="mean", cutoff=4)
+            elif ("__len__" in dir(darks)) and isinstance(darks[0], six.string_types):
+                dark = averageDark([fabio.open(f).data for f in darks if exists(f)], center_method="mean", cutoff=4)
             elif ("__len__" in dir(darks)) and ("ndim" in dir(darks[0])) and (darks[0].ndim == 2):
                 dark = averageDark(darks, center_method="mean", cutoff=4)
         if do_flat and (flat is  None):
             if "ndim" in dir(flats) and flats.ndim == 3:
                 flat = averageDark(flats, center_method="mean", cutoff=4)
-            elif ("__len__" in dir(flats)) and (type(flats[0]) in types.StringTypes):
-                flat = averageDark([fabio.open(f).data for f in flats if os.path.exists(f)], center_method="mean", cutoff=4)
+            elif ("__len__" in dir(flats)) and isinstance(flats[0], six.string_types):
+                flat = averageDark([fabio.open(f).data for f in flats if exists(f)], center_method="mean", cutoff=4)
             elif ("__len__" in dir(flats)) and ("ndim" in dir(flats[0])) and (flats[0].ndim == 2):
                 flat = averageDark(flats, center_method="mean", cutoff=4)
             else:
@@ -516,8 +595,8 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
         if do_dark:
             correctedImg -= dark
         if do_flat:
-             correctedImg /= flat
-        if (cutoff or (filter_ == "median")):
+            correctedImg /= flat
+        if (cutoff or quantiles or (filter_ in ["median", "quantiles"])):
             if big_img is None:
                 logger.info("Big array allocation for median filter or cut-off")
                 big_img = numpy.zeros((ld, shape[0], shape[1]), dtype=numpy.float32)
@@ -537,17 +616,24 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
                 sumImg = correctedImg
             else:
                 sumImg += correctedImg
-    if cutoff or (filter_ == "median"):
-        datared = averageDark(big_img, filter_, cutoff)
+        elif filter_ == "sum":
+            if sumImg is None:
+                sumImg = correctedImg
+            else:
+                sumImg += correctedImg
+    if cutoff or quantiles or (filter_ in ["median", "quantiles"]):
+        datared = averageDark(big_img, filter_, cutoff, quantiles)
     else:
         if filter_ in ["max", "min"]:
             datared = numpy.ascontiguousarray(sumImg, dtype=numpy.float32)
         elif filter_ == "mean":
             datared = sumImg / numpy.float32(ld)
+        elif filter_ == "sum":
+            datared = sumImg / numpy.float32(ld)
     logger.debug("Intensity range in merged dataset : %s --> %s", datared.min(), datared.max())
-    if format is not None:
-        if format.startswith("."):
-            format = format.lstrip(".")
+    if fformat is not None:
+        if fformat.startswith("."):
+            fformat = fformat.lstrip(".")
         if (output is None):
             prefix = ""
             for ch in zip(*listImages):
@@ -565,21 +651,22 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
                 else:
                     break
             if filter_ == "max":
-                output = "maxfilt%02i-%s.%s" % (ld, prefix, format)
+                output = "maxfilt%02i-%s.%s" % (ld, prefix, fformat)
             elif filter_ == "median":
-                output = "medfilt%02i-%s.%s" % (ld, prefix, format)
+                output = "medfilt%02i-%s.%s" % (ld, prefix, fformat)
             elif filter_ == "median":
-                output = "meanfilt%02i-%s.%s" % (ld, prefix, format)
+                output = "meanfilt%02i-%s.%s" % (ld, prefix, fformat)
             else:
-                output = "merged%02i-%s.%s" % (ld, prefix, format)
-        if format and output:
-            if "." in format:  # in case "edf.gz"
-                format = format.split(".")[0]
-            fabiomod = fabio.__getattribute__(format + "image")
-            fabioclass = fabiomod.__getattribute__(format + "image")
+                output = "merged%02i-%s.%s" % (ld, prefix, fformat)
+        if fformat and output:
+            if "." in fformat:  # in case "edf.gz"
+                fformat = fformat.split(".")[0]
+            fabiomod = fabio.__getattribute__(fformat + "image")
+            fabioclass = fabiomod.__getattribute__(fformat + "image")
             header = {"method":filter_,
                       "nframes":ld,
-                      "cutoff":str(cutoff)}
+                      "cutoff":str(cutoff),
+                      "quantiles": str(quantiles)}
             form = "merged_file_%%0%ii" % len(str(len(listImages)))
             header_list = ["method", "nframes", "cutoff"]
             for i, f in enumerate(listImages):
@@ -591,6 +678,10 @@ def averageImages(listImages, output=None, threshold=0.1, minimum=None, maximum=
 #            if "header_keys" in dir(fimg):
             fimg.header_keys = header_list
 
+            if filter_ == "sum":
+                fimg = fabioclass(data=numpy.int32(datared * numpy.float32(ld)),
+                                  header=header)
+
             fimg.write(output)
             logger.info("Wrote %s" % output)
         return output
@@ -650,7 +741,7 @@ def removeSaturatedPixel(ds, threshold=0.1, minimum=None, maximum=None):
         if  minimum is True:  # automatic guess of the best minimum TODO: use the HWHM to guess the minumum...
             data_min = ds.min()
             x, y = numpy.histogram(numpy.log(ds - data_min + 1.0), bins=100)
-            f = interp1d((y[1:] + y[:-1]) / 2.0, -x, bounds_error=False, fill_value= -x.min())
+            f = interp1d((y[1:] + y[:-1]) / 2.0, -x, bounds_error=False, fill_value=-x.min())
             max_low = fmin(f, y[1], disp=0)
             max_hi = fmin(f, y[-1], disp=0)
             if max_hi > max_low:
@@ -684,10 +775,11 @@ def removeSaturatedPixel(ds, threshold=0.1, minimum=None, maximum=None):
     return ds
 
 
-def binning(input_img, binsize):
+def binning(input_img, binsize, norm=True):
     """
     @param input_img: input ndarray
     @param binsize: int or 2-tuple representing the size of the binning
+    @param norm: if False, do average instead of sum
     @return: binned input ndarray
     """
     inputSize = input_img.shape
@@ -701,13 +793,15 @@ def binning(input_img, binsize):
 
     if numpy.array(binsize).prod() < 50:
         out = numpy.zeros(tuple(outputSize))
-        for i in xrange(binsize[0]):
-            for j in xrange(binsize[1]):
+        for i in range(binsize[0]):
+            for j in range(binsize[1]):
                 out += input_img[i::binsize[0], j::binsize[1]]
     else:
         temp = input_img.copy()
         temp.shape = (outputSize[0], binsize[0], outputSize[1], binsize[1])
         out = temp.sum(axis=3).sum(axis=1)
+    if not norm:
+        out /= binsize[0] * binsize[1]
     return out
 
 
@@ -715,6 +809,7 @@ def unBinning(binnedArray, binsize, norm=True):
     """
     @param binnedArray: input ndarray
     @param binsize: 2-tuple representing the size of the binning
+    @param norm: if True (default) decrease the intensity by binning factor. If False, it is non-conservative
     @return: unBinned input ndarray
     """
     if isinstance(binsize, int):
@@ -723,8 +818,8 @@ def unBinning(binnedArray, binsize, norm=True):
     for i, j in zip(binnedArray.shape, binsize):
         outputShape.append(i * j)
     out = numpy.zeros(tuple(outputShape), dtype=binnedArray.dtype)
-    for i in xrange(binsize[0]):
-        for j in xrange(binsize[1]):
+    for i in range(binsize[0]):
+        for j in range(binsize[1]):
             out[i::binsize[0], j::binsize[1]] += binnedArray
     if norm:
         out /= binsize[0] * binsize[1]
@@ -772,6 +867,7 @@ def shiftFFT(input_img, shift_val, method="fftw"):
         out = numpy.fft.ifft2(numpy.fft.fft2(input_img) * e)
     return abs(out)
 
+
 def maximum_position(img):
     """
     Same as scipy.ndimage.measurements.maximum_position:
@@ -784,6 +880,7 @@ def maximum_position(img):
     _, s1 = img.shape
     return (maxarg // s1, maxarg % s1)
 
+
 def center_of_mass(img):
     """
     Calculate the center of mass of of the array.
@@ -797,6 +894,7 @@ def center_of_mass(img):
     img /= img.sum()
     return ((a0 * img).sum(), (a1 * img).sum())
 
+
 def measure_offset(img1, img2, method="numpy", withLog=False, withCorr=False):
     """
     Measure the actual offset between 2 images
@@ -889,7 +987,7 @@ def expand_args(args):
     """
     new = []
     for afile in  args:
-        if os.path.exists(afile):
+        if exists(afile):
             new.append(afile)
         else:
             new += glob.glob(afile)
@@ -913,14 +1011,6 @@ def _get_data_path(filename):
 
     For now, just perform a recursive search
     """
-    # when using bootstrap the file is located under the build directory
-#    real_filename = os.path.abspath(os.path.join(os.path.dirname(__file__),
-#                                                 os.path.pardir,
-#                                                 os.path.pardir,
-#                                                 os.path.pardir,
-#                                                 'data',
-#                                                 filename))
-#    if not os.path.exists(real_filename):
     resources = [os.environ.get("PYFAI_DATA"), data_dir, os.path.dirname(__file__)]
     try:
         import xdg.BaseDirectory
@@ -939,14 +1029,39 @@ def _get_data_path(filename):
                         " something went wrong !!!" % (real_filename,))
 
 
-def get_ui_file(filename):
-    return _get_data_path(os.path.join("gui", filename))
-#    return _get_data_path(filename)
+def get_calibration_dir():
+    """get the full path of a calibration directory
+
+    @return: the full path of the calibrant file
+    """
+    return _get_data_path("calibration")
 
 
 def get_cl_file(filename):
-#    return _get_data_path(os.path.join("openCL", filename))
-    return _get_data_path(filename)
+    """get the full path of a openCL file
+
+    @return: the full path of the openCL source file
+    """
+    return _get_data_path(os.path.join("openCL", filename))
+
+
+def get_ui_file(filename):
+    """get the full path of a user-interface file
+
+    @return: the full path of the ui
+    """
+    return _get_data_path(os.path.join("gui", filename))
+
+
+def read_cl_file(filename):
+    """
+    @param filename: read an OpenCL file and apply a preprocessor
+    @return: preprocessed source code
+    """
+    with open(get_cl_file(filename), "r") as f:
+        # Dummy preprocessor which removes the #include
+        lines = [i for i in f.readlines() if not i.startswith("#include ")]
+    return "".join(lines)
 
 
 def concatenate_cl_kernel(filenames):
@@ -958,9 +1073,8 @@ def concatenate_cl_kernel(filenames):
     """
     kernel = ""
     for filename in filenames:
-        with open(get_cl_file(filename), "r") as f:
-            kernel += f.read()
-
+            kernel += read_cl_file(filename)
+            kernel += os.linesep
     return kernel
 
 
@@ -975,10 +1089,11 @@ def deg2rad(dd):
     """
     while dd > 180.0:
         dd -= 360.0
-    while dd <= -180.0:
+    while dd < -180.0:
         dd += 360.0
     return dd * pi / 180.
 
+
 class lazy_property(object):
     '''
     meant to be used for lazy evaluation of an object attribute.
@@ -987,8 +1102,7 @@ class lazy_property(object):
 
     def __init__(self, fget):
         self.fget = fget
-        self.func_name = fget.__name__
-
+        self.func_name = fget.func_name if sys.version_info[0] < 3 else fget.__name__
 
     def __get__(self, obj, cls):
         if obj is None:
@@ -999,7 +1113,7 @@ class lazy_property(object):
 
 try:
     from numpy import percentile
-except ImportError: #backport percentile from numpy 1.6.2
+except ImportError:  # backport percentile from numpy 1.6.2
     np = numpy
     def percentile(a, q, axis=None, out=None, overwrite_input=False):
         """
@@ -1113,7 +1227,7 @@ except ImportError: #backport percentile from numpy 1.6.2
 
         q = q / 100.0
         if (q < 0) or (q > 1):
-            raise ValueError, "percentile must be either in the range [0,100]"
+            raise ValueError("percentile must be either in the range [0,100]")
 
         indexer = [slice(None)] * sorted.ndim
         Nx = sorted.shape[axis]
@@ -1136,6 +1250,7 @@ except ImportError: #backport percentile from numpy 1.6.2
         #   check and use out array.
         return np.add.reduce(sorted[indexer] * weights, axis=axis, out=out) / sumval
 
+
 def convert_CamelCase(name):
     """
     convert a function name in CamelCase into camel_case
@@ -1144,6 +1259,7 @@ def convert_CamelCase(name):
     s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
     return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
 
+
 def readFloatFromKeyboard(text, dictVar):
     """
     Read float from the keyboard ....
@@ -1165,3 +1281,88 @@ def readFloatFromKeyboard(text, dictVar):
                     dictVar[i][j](vals[j])
         if not found:
             logger.error("You should provide the good number of floats")
+
+
+class FixedParameters(set):
+    """
+    Like a set, made for FixedParameters in geometry refinement
+    """
+
+    def add_or_discard(self, key, value=True):
+        """
+        Add a value to a set if value, else discard it
+        @param key: element to added or discared from set
+        @type value: boolean. If None do nothing !
+        @return: None
+        """
+        if value is None:
+            return
+        if value:
+            self.add(key)
+        else:
+            self.discard(key)
+
+
+def roundfft(N):
+    """
+    This function returns the integer >=N for which size the Fourier analysis is faster (fron the FFT point of view)
+    Credit: Alessandro Mirone, ESRF, 2012
+
+    @param N: interger on which one would like to do a Fourier transform
+    @return: integer with a better choice
+    """
+    MA, MB, MC, MD, ME, MF = 0, 0, 0, 0, 0, 0
+    FA, FB, FC, FD, FE, FFF = 2, 3, 5, 7, 11, 13
+    DIFF = 9999999999
+    RES = 1
+    R0 = 1
+    AA = 1
+    for A in range(int(math.log(N) / math.log(FA) + 2)):
+        BB = AA
+        for B in range(int(math.log(N) / math.log(FB) + 2)):
+            CC = BB
+
+            for C in range(int(math.log(N) / math.log(FC) + 2)):
+                DD = CC
+
+                for D in range(int(math.log(N) / math.log(FD) + 2)):
+                    EE = DD
+
+                    for E in range(2):
+                        FF = EE
+
+                        for F in range(2 - E):
+                            if FF >= N and DIFF > abs(N - FF):
+                                MA, MB, MC, MD, ME, MF = A, B, C, D, E, F
+                                DIFF = abs(N - FF)
+                                RES = FF
+                            if FF > N: break
+                            FF = FF * FFF
+                        if EE > N: break
+                        EE = EE * FE
+                    if DD > N: break
+                    DD = DD * FD
+                if CC > N: break
+                CC = CC * FC
+            if BB > N: break
+            BB = BB * FB
+        if AA > N: break
+        AA = AA * FA
+    return RES
+
+
+def is_far_from_group(pt, lst_pts, d2):
+    """
+    Tells if a point is far from a group of points, distance greater than d2 (distance squared)
+
+    @param pt: point of interest
+    @param lst_pts: list of points
+    @param d2: minimum distance squarred
+    @return: True If the point is far from all others.
+
+    """
+    for apt in lst_pts:
+        dsq = sum((i - j) * (i - j) for i, j in zip(apt, pt))
+        if dsq <= d2:
+            return False
+    return True
