@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #    Project: Fast Azimuthal integration
-#             https://github.com/kif/pyFAI
+#             https://github.com/pyFAI/pyFAI
 #
 #    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
 #
@@ -28,7 +28,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "GPLv3+"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "22/09/2014"
+__date__ = "22/06/2015"
 __status__ = "development"
 __doc__ = """
 
@@ -39,10 +39,10 @@ additional saving capabilities like
 - save as 2/3D structure in a HDF5 File
 - read from HDF5 files
 
-Aims at being integrated into a plugin like LImA or as model for the GUI 
+Aims at being integrated into a plugin like LImA or as model for the GUI
 
 The configuration of this class is mainly done via a dictionary transmitted as a JSON string:
-Here are the valid keys:  
+Here are the valid keys:
     "dist",
     "poni1",
     "poni2",
@@ -65,7 +65,7 @@ Here are the valid keys:
     "do_2D"
     "azimuth_range_min"
     "azimuth_range_max"
-     
+
     "polarization_factor"
     "nbpt_rad"
     "do_solid_angle"
@@ -95,12 +95,15 @@ import numpy
 import fabio
 from .detectors import detector_factory
 from .azimuthalIntegrator import AzimuthalIntegrator
+from .distortion import Distortion
 from . import units
 import json
-#from .io import h5py, HDF5Writer
+# from .io import h5py, HDF5Writer
 
 class Worker(object):
-    def __init__(self, azimuthalIntgrator=None, shapeIn=(2048, 2048), shapeOut=(360, 500), unit="r_mm"):
+    def __init__(self, azimuthalIntgrator=None,
+                 shapeIn=(2048, 2048), shapeOut=(360, 500),
+                 unit="r_mm", dummy=None, delta_dummy=None):
         """
         @param azimuthalIntgrator: pyFAI.AzimuthalIntegrator instance
         @param shapeIn: image size in input
@@ -125,12 +128,12 @@ class Worker(object):
 #                self.config = json.loads(config)
 #        if self.config:
 #            self.configure()
-        self._normalization_factor = None #Value of the monitor: divides the intensity by this value for normalization
+        self._normalization_factor = None  # Value of the monitor: divides the intensity by this value for normalization
         self.nbpt_azim, self.nbpt_rad = shapeOut
         self._unit = units.to_unit(unit)
         self.polarization = None
-        self.dummy = None
-        self.delta_dummy = None
+        self.dummy = dummy
+        self.delta_dummy = delta_dummy
         self.correct_solid_angle = True
         self.dark_current_image = None
         self.flat_field_image = None
@@ -139,7 +142,7 @@ class Worker(object):
         self.extension = None
         self.do_poisson = None
         self.needs_reset = True
-        self.output = "numpy" #exports as numpy array by default
+        self.output = "numpy"  # exports as numpy array by default
         self.shape = shapeIn
         self.method = "lut"
         self.radial = None
@@ -180,7 +183,7 @@ class Worker(object):
     def reconfig(self, shape=(2048, 2048), sync=False):
         """
         This is just to force the integrator to initialize with a given input image shape
-        
+
         @param shape: shape of the input image
         @param sync: return only when synchronized
         """
@@ -191,12 +194,12 @@ class Worker(object):
     def process(self, data) :
         """
         Process a frame
-        #TODO: 
+        #TODO:
         dark, flat, sa are missing
-        
+
         @param: data: numpy array containing the input image
         """
-        
+
         with self._sem:
             monitor = self._normalization_factor
         kwarg = {"unit": self.unit,
@@ -207,7 +210,8 @@ class Worker(object):
                  # "filename": None,
                  "safe": True,
                  "data": data,
-                 "normalization_factor":monitor
+                 "normalization_factor":monitor,
+                 "correctSolidAngle": self.correct_solid_angle,
                  }
 
 
@@ -231,22 +235,28 @@ class Worker(object):
         else:
             kwarg["error_model"] = None
 
-#        try:
-        if 1:
+        try:
+#         if 1:
             if self.do_2D():
                 rData, self.radial, self.azimuthal = self.ai.integrate2d(**kwarg)
             else:
-
                 rData = self.ai.integrate1d(**kwarg)
                 self.radial = rData[0]
                 rData = numpy.vstack(rData).T
 
-#        except:
-#            print(data.shape, data.size)
-#            print(self.ai)
-#            print(self.ai._lut_integrator)
-#            print(self.ai._lut_integrator.size)
-#            raise
+        except Exception as err:
+            err2 = ["error in integration",
+                    str(err),
+                    "data.shape: %s" % data.shape,
+                    "data.size: %s" % data.size,
+                    "ai:",
+                    str(self.ai),
+                    "csr:",
+                    str(self.ai._csr_integrator),
+                    "csr size: %s" % self.ai._lut_integrator.size]
+            logger.error(err2)
+            raise err
+
         if self.output == "numpy":
             return rData
 
@@ -359,19 +369,19 @@ class Worker(object):
     def get_config(self):
         """return configuration as a dictionary"""
         config = {"unit":str(self.unit)}
-        for key in ["dist",    "poni1",    "poni2",    "rot1",    "rot3",    "rot2",    "pixel1",    "pixel2",    "splineFile",    "wavelength"]:
+        for key in ["dist", "poni1", "poni2", "rot1", "rot3", "rot2", "pixel1", "pixel2", "splineFile", "wavelength"]:
             try:
                 config[key] = self.ai.__getattribute__(key)
             except:
                 pass
         for key in ["nbpt_azim", "nbpt_rad", "polarization", "dummy", "delta_dummy", "correct_solid_angle", "dark_current_image", "flat_field_image", "mask_image",
-                  "do_poisson","shape","method"
+                  "do_poisson", "shape", "method"
                   ]:
             try:
                 config[key] = self.__getattribute__(key)
             except:
                 pass
-            
+
         return config
 #
 #    "poni" #path of the file
@@ -403,11 +413,11 @@ class Worker(object):
 #    "val_dummy"
 #    "do_dummy"
 #    "method"
-#}
+# }
 
     def get_json_config(self):
         """return configuration as a JSON string"""
-        pass #TODO
+        pass  # TODO
 
     def save_config(self, filename=None):
         if not filename:
@@ -428,19 +438,23 @@ class Worker(object):
         if sync:
             t.join()
 
+    def get_normalization_factor(self):
+        with self._sem:
+            return self._normalization_factor
     def set_normalization_factor(self, value):
         with self._sem:
             self._normalization_factor = value
+    normalization_factor = property(get_normalization_factor, set_normalization_factor)
 
 
 class PixelwiseWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
-    def __init__(self, dark=None, flat=None, solidangle=None, polarization=None, 
+    def __init__(self, dark=None, flat=None, solidangle=None, polarization=None,
                  mask=None, dummy=None, delta_dummy=None, device=None):
         """
-        @param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context 
+        @param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context
         """
         self.ctx = None
         if dark is not None:
@@ -490,16 +504,17 @@ class PixelwiseWorker(object):
             do_mask = True
         else:
             do_mask = (self.mask is not False)
-        data = numpy.array(data, dtype=numpy.float32) #Explicitely make an copy !
-        if self.dark:
+        # Explicitly make an copy !
+        data = numpy.array(data, dtype=numpy.float32)
+        if self.dark is not None:
             data -= self.dark
-        if self.flat:
+        if self.flat is not None:
             data /= self.flat
-        if self.solidangle:
+        if self.solidangle is not None:
             data /= self.solidangle
-        if self.polarization:
+        if self.polarization is not None:
             data /= self.polarization
-        if normalization:
+        if normalization is not None:
             data /= normalization
         if do_mask:
             data[self.mask] = self.dummy or 0
@@ -510,18 +525,13 @@ class DistortionWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
-    def __init__(self, detector=None, dark=None, flat=None, solidangle=None, polarization=None, 
+    def __init__(self, detector=None, dark=None, flat=None, solidangle=None, polarization=None,
                  mask=None, dummy=None, delta_dummy=None, device=None):
         """
-        @param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context 
+        @param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context
         """
-        
+
         self.ctx = None
-        if detector is None:
-            self.distortion = None
-        else:
-            self.distortion = pyFAI.distortion()
-            #TODO
         if dark is not None:
             self.dark = numpy.ascontiguousarray(dark, dtype=numpy.float32)
         else:
@@ -539,6 +549,11 @@ class DistortionWorker(object):
         else:
             self.polarization = None
 
+        if detector is None:
+            self.distortion = None
+        else:
+            self.distortion = Distortion(detector, method="CSR", device=device)
+
         if mask is None:
             self.mask = False
         elif mask.min() < 0 and mask.max() == 0:  # 0 is valid, <0 is invalid
@@ -569,19 +584,23 @@ class DistortionWorker(object):
             do_mask = True
         else:
             do_mask = (self.mask is not False)
-        data = numpy.array(data, dtype=numpy.float32) #Explicitely make an copy !
-        if self.dark:
+        # Explicitly make an copy !
+        data = numpy.array(data, dtype=numpy.float32)
+        if self.dark is not None:
             data -= self.dark
-        if self.flat:
+        if self.flat is not None:
             data /= self.flat
-        if self.solidangle:
+        if self.solidangle is not None:
             data /= self.solidangle
-        if self.polarization:
+        if self.polarization is not None:
             data /= self.polarization
-        
+
         if do_mask:
             data[self.mask] = self.dummy or 0
-        return data
+        if self.distortion is not None:
+            return self.distortion.correct(data)
+        else:
+            return data
 
 
 
